﻿var app = angular.module('fsrTopTen', [
  'ngRoute'
 , 'ngAria'
 , 'ngAnimate'
 , 'ngMaterial'
 , 'onsen'
 , 'bsHas' // bootstrap error directives
 , 'nxValidators' // bootstrap error directives
 , 'pascalprecht.translate'
 , 'tmh.dynamicLocale'
 , 'fsrTopTenFilters'
 , 'fsrTopTenComponents'
 , 'fsrTopTenDirectives'
 , 'fsrTopTenControllers'
 , 'fsrTopTenServices'
 , 'fsrTopTenInterceptorServices'
 , 'angularUtils.directives.dirPagination'
 , 'nsPopover'
 , 'chart.js'
 , 'angulartics'
 , 'angulartics.google.analytics'
 , 'ngStorage'
]);

// hack to make angular material work correctly
app.config(['$mdGestureProvider', function ($mdGestureProvider) {
    $mdGestureProvider.skipClickHijack();
}]);

app.config(['$httpProvider', '$routeProvider', '$translateProvider', '$analyticsProvider', 'tmhDynamicLocaleProvider', '$provide', '$logProvider', '$locationProvider', 'appSettings',
    function ($httpProvider, $routeProvider, $translateProvider, $analyticsProvider, tmhDynamicLocaleProvider, $provide, $logProvider, $locationProvider, appSettings) {

    $logProvider.debugEnabled(appSettings.DEBUG);

    $httpProvider.interceptors.push('AuthInterceptorService');

	$translateProvider.useStaticFilesLoader({ prefix: 'js/libs/l10n/languages/', suffix: '.json' });
	$translateProvider.preferredLanguage('eng');
	$translateProvider.fallbackLanguage('eng');

	tmhDynamicLocaleProvider.localeLocationPattern('js/libs/angular/i18n/angular-locale_{{locale}}.js');

	$analyticsProvider.virtualPageviews(true);
	$analyticsProvider.firstPageview(true);

    $routeProvider.
		when('/admin', {
			templateUrl: 'partials/admin/index.html',
			resolve: {
			    screen: ["$q", "$rootScope", function ($q, $rootScope) {
			        $rootScope.isFullScreen = true;
			        $rootScope.hideNavbar = false;
				}]
			}
		}).
		//when('/counter/:startDate', {
		when('/counter/:raceId/:startDate', {
		    templateUrl: 'partials/counter/index.html',
		    resolve: {
		        screen: ["$q", "$rootScope", function ($q, $rootScope) {
		            $rootScope.isFullScreen = true;
		            $rootScope.hideNavbar = true;
		        }]
		    }
		}).
		when('/login', {
		    templateUrl: 'partials/login/start.html',
		    resolve: {
		        screen: ["$q", "$rootScope", function ($q, $rootScope) {
		            $rootScope.isFullScreen = false;
		            $rootScope.hideNavbar = false;
		        }]
		    }
		}).
        when('/dashboard/:tab?', {
            templateUrl: 'partials/dashboard/index.html',
            reloadOnSearch: false,
            resolve: {
                screen: ["$q", "$rootScope", function ($q, $rootScope) {
                    $rootScope.isFullScreen = false;
                    $rootScope.hideNavbar = false;
                }],
                auth: ["$q", "AuthService", function ($q, AuthService) {
                    var userInfo = AuthService.getAuthentication();
                    if (userInfo.isAuth) {
                        return $q.when(userInfo);
                    } else {
                        return $q.reject({ authenticated: false });
                    }
                }],
                state: ["AppStateService", "$route", "$location", function (AppStateService, $route, $location) {
                    var tab = $route.current.params.tab;
                    var id = $location.search().id;
                    var tabIndex = -1;
                    switch (tab) {
                        case "leagues":
                            tabIndex = 0;
                            id = parseInt(id);
                            AppStateService.setLeagueId(id);
                            break;
                        case "races":
                            tabIndex = 1;
                            id = parseInt(id);
                            break;
                        case "challenges":
                            tabIndex = 2;
                            id = parseInt(id);
                            break;
                        case "social":
                            tabIndex = 3;
                            id = id;
                            break;
                        case "settings":
                            tabIndex = 4;
                            id = null;
                            break;
                    }
                    if (tabIndex >= 0) {
                        AppStateService.setTab(tabIndex);
                        AppStateService.setTabContext(id);
                    }
                    return {};
                }]
            }
        }).
		when('/blog', {
		    templateUrl: 'partials/home/blog.html',
		    resolve: {
		        screen: ["$rootScope", function ($rootScope) {
		            $rootScope.isFullScreen = false;
		            $rootScope.hideNavbar = false;
		        }]
		    }
		}).
        // removed for now since the landing page isn't production-worthy
        /*
		when('/', {
			templateUrl: 'partials/home/index.html'
		}).
        */
		otherwise({
			redirectTo: '/dashboard'
		});
}]);

app.run(["$rootScope", "$location", "AppStateService", "$analytics", "sportContextService", "navService", function ($rootScope, $location, AppStateService, $analytics, sportContextService, navService) {

    // TODO: Set this up to some account-level default by API call? 
    if (sportContextService.getContext() === '') {
        sportContextService.setContext('AL');
    }

    if (navService && !navService.isInitialized()) {
        if (typeof (navService.initialize) === "function")
        {
            navService.initialize();
        }
    }

    $rootScope.$on("$routeUpdate", function (event, current) {

        $analytics.pageTrack($location.url());

        var search = $location.search();
        var newDepth = 0;
        if (search) {
            if (search.d) {
                var currentDepth = AppStateService.getDepth();
                newDepth =  parseInt(search.d);
                if (newDepth < currentDepth) {
                    if ($rootScope.getNavigator().pages.length > 1) {
                        $rootScope.getNavigator().popPage();
                    }
                }
                AppStateService.setDepth(newDepth);
            } else {
                if (AppStateService.getDepth() > 0) {
                    if ($rootScope.getNavigator().pages.length > 1) {
                        $rootScope.getNavigator().popPage();
                    }
                }

                if (!search.id) {
                    if ($rootScope.getNavigator().pages.length > 1) {
                        $rootScope.getNavigator().popPage();
                    }
                }
            }

        }
        AppStateService.setDepth(newDepth);
    });

	$rootScope.$on("$routeChangeSuccess", function (userInfo) {
	    //console.log("route change");
	    AppStateService.setDepth(0);
	});

	$rootScope.$on("$routeChangeError", function (event, current, previous, eventObj) {
		if (eventObj.authenticated === false) {
			$location.path("/login");
		}
		AppStateService.setDepth(0);
	});
}]);

app.run(["AnalyticsService", "AuthService", function (AnalyticsService, AuthService) {
	var userInfo = AuthService.getUserData();
	AnalyticsService.setUsername(userInfo);


	if (typeof cordova != 'undefined' && typeof device != 'undefined') {
	    ga('create', 'UA-27378356-1', {
	        'storage': 'none',
	        'clientId': device.uuid
	    });
	    ga('set', 'checkProtocolTask', function (data) {
	        data.set('location', 'https://app.fantasyskiracer.com');
	    });
	}
	else {
	    ga('create', 'UA-27378356-1', 'auto');
	}
}]);

app.run(["LocalNotificationService", function (LocalNotificationService) {
    LocalNotificationService.Init();
}]);
