﻿// Contains several controllers -- 

// Account
// -> Edit
// -> UpdatePassword
// Scoring
// USSA

module.controller('AccountCtrl', ['$rootScope', '$scope', '$routeParams', '$translate', '$log', 'dataService', 'AuthService', 'AccountService', 'TranslationService', 'navService', function ($rootScope, $scope, $routeParams, $translate, $log, dataService, AuthService, AccountService, TranslationService, navService) {

    navService.initializeStack({ title: $translate.instant('Settings'), controlContext: false });

	$scope.navigateEdit = function () {
	    $rootScope.pushPage('partials/settings/account/edit.html', {});
	    navService.push({ title: $translate.instant('Edit Profile'), controlContext: false });
	};

	$scope.navigateChangePassword = function () {
	    $rootScope.pushPage('partials/settings/account/update-password.html', {});
	    navService.push({ title: $translate.instant('Change Password'), controlContext: false });
	};

	$scope.navigateScoring = function () {
	    $rootScope.pushPage('partials/settings/scoring.html', {});
	    navService.push({ title: $translate.instant('Scoring Details'), controlContext: false });
	};

	$scope.navigateUSSA = function () {
	    $rootScope.pushPage('partials/settings/ussa.html', {});
	    navService.push({ title: $translate.instant('USSA Contest Details'), controlContext: false });
	};

	$scope.logout = function () {
	    AuthService.logout();
	};

	$scope.openBrowser = function (url) {
		if ($rootScope.isMobile) {
			//in native use system browser
			window.open(url, '_system');
		}
		else {
			//open in new tab
			window.open(url);
		}
	};

}]);

module.controller('EditCtrl', ['$rootScope', '$scope', '$routeParams', '$translate', '$log', 'dataService', 'AuthService', 'AccountService', 'TranslationService', 'navService', function ($rootScope, $scope, $routeParams, $translate, $log, dataService, AuthService, AccountService, TranslationService, navService) {

    $scope.account = { username: '', email: '', country: '', language: '', name: '', gender: '', dob: '', ussa: '' };

    $scope.result = {
        success: false,
        message: ""
    };

    $scope.select = { countries: [], languages: [], clubs: [] };
    $scope.error = { message: '' };

    TranslationService.getLanguages().then(function (response) {
        $scope.select.languages = response.data;
    }, function (err) {
    });

    TranslationService.getCountries().then(function (response) {
        $scope.select.countries = response.data;
    }, function (err) {
    });

    $scope.username = AuthService.getAuthentication().username;

    $scope.getAccount = function () {
        $rootScope.loading++;
        AccountService.get({ username: $scope.username },
            function (response) {
                $rootScope.loading--;

                //quick formatting tweaks for displaying correctly on the page
                response.name = response.name || "";
                response.gender = response.gender || "";
                response.ussa = response.ussa === "0" ? "" : response.ussa;
                response.dob = response.dob ? new Date(response.dob) : null;
                response.club_id = response.club_id ? response.club_id : -1;
                $scope.account = response;

                // do this once to fill the clubs

                clubSearch();
            },
            function (err) {
                $rootScope.loading--;
                $rootScope.getNavigator().popPage();
            });
    };

    $scope.updateAccount = function () {
        $scope.submitted = true;
        $scope.result.message = "";
        if ($scope.form.$invalid) {
            $scope.result.success = false;
            $scope.result.message = "Missing required information.";
            return;
        }

        AccountService.update($scope.account,
            function (response) {
                if (TranslationService.getLanguage() !== $scope.account.language) {
                    TranslationService.changeLanguage($scope.account.language);
                }
            },
            function (err) {
                $scope.result.success = false;
                $scope.result.message = err.data.Message;
            });
    };

    var THROTTLE = 500; // throttle to only search after 500ms of inactivity
    var lastSearch;
    var nextSearchFn;
    $scope.findRelatedClubs = function () {
        if (lastSearch && (new Date() - lastSearch < THROTTLE)) {
            // if there is already a search, cancel it
            if (nextSearchFn) clearTimeout(nextSearchFn);

            // set the next search
            nextSearchFn = setTimeout(function () {
                clubSearch();
            }, THROTTLE);
        }
        else {
            clubSearch();
        }
    }

    function clubSearch() {
        if ($scope.account.ussa !== "") {
            dataService.Club.getByUssaId({ id: $scope.account.ussa }, function success(data) {
                $scope.select.clubs = data;
            }, function error(err) {
                $log.debug('Err:' + JSON.stringify(err));
                $scope.error.message = err.data ? err.data.Message : 'Error Searching';
            });
        }

        lastSearch = new Date();
    }

}]);

module.controller('UpdatePasswordCtrl', ['$rootScope', '$scope', '$routeParams', '$translate', '$log', 'dataService', 'AuthService', 'AccountService', 'TranslationService', 'navService', function ($rootScope, $scope, $routeParams, $translate, $log, dataService, AuthService, AccountService, TranslationService, navService) {

    $scope.result = {
        success: false,
        message: ""
    };

    $scope.changePassword = {
        currentPassword: "",
        newPassword: "",
        newPasswordVerify: ""
    };

    $scope.updatePassword = function () {
        $scope.submitted = true;
        if ($scope.form.$invalid) {
            $scope.result.success = false;
            $scope.result.message = "Missing required information.";
            return;
        }

        $rootScope.loading++;
        AccountService.updatepassword($scope.changePassword,
            function (response) {
                $rootScope.loading--;
                $rootScope.getNavigator().popPage();
            },
            function (err) {
                $rootScope.loading--;
                $scope.result.success = false;
                $scope.result.message = err.data.Message;
            });
    };

}]);

module.controller('ScoringCtrl', ['$rootScope', function ($rootScope) {

}]);

module.controller('USSACtrl', ['$rootScope', function ($rootScope) {

}]);