﻿module.controller('AdminAdCtrl', ['$scope', '$routeParams', '$log', '$window', 'dataService', 'UploadService', function ($scope, $routeParams, $log, $window, dataService, UploadService) {
	$scope.ads = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingAd = {};
	$scope.error = {};
	$scope.searchName = '';

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'ads' && !$scope.loaded) {
				loadAds();
			}
		});

	function loadAds() {
		$scope.pageLoading++;
		dataService.Ad.get(function success(ads) {
			$scope.ads = ads;
			$scope.loaded = true;
			$scope.pageLoading--;
		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Ads';
		});
	};

	//----UI functions----
	$scope.isEditing = function (id) {
		return id == $scope.editingAd.id;
	};

	//listen for the file selected event
	$scope.$on("fileSelected", function (event, args) {
		$scope.$apply(function () {
			$scope.imageType = args.type;
			switch (args.type) {
				case "normal":
					$scope.imageFileNormal = args.file;
					break;
				case "sidebar":
					$scope.imageFileSidebar = args.file;
					break;
				case "background":
					$scope.imageFileBackground = args.file;
					break;
				case "loading":
					$scope.imageFileLoading = args.file;
					break;
			}
		});

		// update the image for preview
		var reader = new FileReader();
		reader.onload = function (e) {
			$scope.$apply(function () {
				switch ($scope.imageType) {
					case "normal":
						$scope.editingAd.normal_preview_path = e.target.result;
						break;
					case "sidebar":
						$scope.editingAd.sidebar_preview_path = e.target.result;
						break;
					case "background":
						$scope.editingAd.background_preview_path = e.target.result;
						break;
					case "loading":
						$scope.editingAd.loading_preview_path = e.target.result;
						break;
				}
			});
		}
		reader.readAsDataURL(args.file);
	});

	$scope.edit = function (ad) {
		if ($scope.isEditing(ad.id)) {
			// if I'm already editing the ad and I click on 'edit' again, then clear it and close
			$scope.reset();
		}
		else {
			// only set the edit if I'm not already editing it (or I'm editing a different ad)
			$scope.reset();
			$scope.editingAd = angular.copy(ad);
			if ($scope.editingAd.id == 0) {
				$scope.editingAd.active = true;
			}
			$scope.imageFileNormal = null;
			$scope.imageFileSidebar = null;
			$scope.imageFileBackground = null;
			$scope.imageFileLoading = null;
		}
	};

	$scope.save = function (ad) {
		// fix the object to be what the service expects
		delete ad.normal_preview_path;
		delete ad.background_preview_path;
		delete ad.loading_preview_path;
		delete ad.sidebar_preview_path;
		if (ad.id == 0) {
			// for a new ad, pre-populate some fields that will get set later, on image upload
			ad.image_path = "";
		}

		$scope.pageLoading++;
		dataService.Ad.save(ad, function success(savedAd) {
			// now save the image
			if ($scope.imageFileNormal || $scope.imageFileSidebar || $scope.imageFileBackground || $scope.imageFileLoading) {
				UploadService.saveAdImages(ad.id, $scope.imageFileNormal, $scope.imageFileSidebar, $scope.imageFileBackground, $scope.imageFileLoading).then(
					function (data) {
						savedAd.image_path = data.image_path;
						savedAd.sidebar_image_path = data.sidebar_image_path;
						savedAd.background_image_path = data.background_image_path;
						savedAd.loading_image_path = data.loading_image_path;
					},
					function (message) {
						alert(message);
					});
			}

			if (ad.id == 0) { // add new mode
				$scope.ads.push(savedAd);
			}
			else {  // edit existing mode
				angular.forEach($scope.ads, function (value, key) {
					if (value.id == savedAd.id) {
						$scope.ads[key] = savedAd;
					}
				});
			}

			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Ad';
		});
	}

	$scope.delete = function (ad) {
		var deleteAd = $window.confirm('Are you sure you want to delete this ad?');

		if (deleteAd) {
			$scope.pageLoading++;
			dataService.Ad.delete({ id: ad.id }, function success() {
				var index = $scope.ads.indexOf(ad);
				$scope.ads.splice(index, 1);

				$scope.reset();
				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Deleting Ad';
			});
		}
	};

	$scope.reset = function () {
		$scope.editingAd = { };
	};
}]);