﻿module.controller('AdminAthleteCtrl', ['$scope', '$routeParams', '$log', '$translate', 'dataService', 'TranslationService', function ($scope, $routeParams, $log, $translate, dataService, TranslationService) {
	$scope.athletes = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingAthlete = {};
	$scope.error = {};
	$scope.select = { countries: [], disciplines: [], equipments: [], socials: [] };
	$scope.sortType = 'name';
	$scope.sortReverse = false;
	$scope.searchName = '';

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'athletes' && !$scope.loaded) {
			    loadAthletes();
			    loadDisciplines();
			    loadSocials();
			    loadEquipments();
			}
		});

	TranslationService.getCountries().then(function (response) {
		$scope.select.countries = response.data;
	}, function (err) {
	});

	function loadDisciplines() {
	    $scope.pageLoading++;
	    dataService.Admin.disciplines(function success(disciplines) {
	        $scope.select.disciplines = disciplines.map(function (item) {
	            return { id: item, text: $translate.instant(item) }
	        });
	        $scope.pageLoading--;
	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Disciplines';
	    });
	};

	function loadEquipments() {
	    $scope.pageLoading++;
	    dataService.Admin.equipments(function success(data) {
	        $scope.select.equipments = data;
	        $scope.pageLoading--;
	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Equipment';
	    });
	};

	function loadSocials() {
	    $scope.pageLoading++;
	    dataService.Admin.socials(function success(data) {
	        $scope.select.socials = data;
	        $scope.pageLoading--;
	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Social';
	    });
	};

	function loadAthletes() {
		$scope.pageLoading++;
		dataService.Athlete.query(function success(athletes) {
			angular.forEach(athletes, function (value, index) {
				value = convertDisciplines(value);
				value.bgcolor = value.active ? "white" : "lightgray";
				value.dob = new Date(value.dob);
			});
			$scope.athletes = athletes;
			$scope.loaded = true;
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Athletes';
		});
	};

	function convertDisciplines(athlete) {
	    //converts the disciplines that came from the API call and converts them to disciplines that the UI can actually use
	    var disciplines = {};
	    athlete.disciplines.forEach(function(discipline) {
	        disciplines[discipline.discipline_id] = $.grep(athlete.disciplines, function (e) { return e.type == discipline.discipline_id; })[0] || { type: discipline.discipline_id, rank: "" };
	    });
	    athlete.disciplines = disciplines;

		return athlete;
	}

	//----UI functions----
	$scope.changeSort = function (sortType) {
		if ($scope.sortType == sortType) {
			$scope.sortReverse = !$scope.sortReverse
		}
		else {
			$scope.sortReverse = false;
		}
		$scope.sortType = sortType;
	};

	$scope.isEditing = function (id) {
		return id == $scope.editingAthlete.id;
	};

	$scope.edit = function (athlete) {
		$scope.reset();
		$scope.editingAthlete = angular.copy(athlete);
	};

	$scope.delete = function (athlete) {
		$scope.pageLoading++;
		dataService.Athlete.delete({ id: athlete.id }, function success() {
			/* don't delete, just mark inactive */
			//var index = $scope.athletes.indexOf(athlete);
			//$scope.athletes.splice(index, 1);

			angular.forEach($scope.athletes, function (value, key) {
				if (value.id == athlete.id) {
					$scope.athletes[key].active = false;
					$scope.athletes[key].bgcolor = "lightgray";
				}
			});

			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Deleting Athlete';
		});
	};

	$scope.reset = function () {
		$scope.editingAthlete = {};
	};

	$scope.save = function (athlete) {
		$scope.pageLoading++;

		dataService.Athlete.save(athlete, function success(savedAthlete) {
			if (athlete.id == 0) {
				$scope.athletes.push(savedAthlete);
			}
			else {
				angular.forEach($scope.athletes, function (value, key) {
				    if (value.id == savedAthlete.id) {
                        // update to the already converted disciplines (since it can't change as part of editing athlete anyway)
				        savedAthlete.disciplines = athlete.disciplines;
				        savedAthlete.dob = new Date(savedAthlete.dob);
						$scope.athletes[key] = savedAthlete;
						$scope.athletes[key].bgcolor = savedAthlete.active ? "white" : "lightgray";
					}
				});
			}
			
			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Athlete';
		});
	};

	$scope.updateRankings = function () {
	    $scope.pageLoading++;
	    dataService.Admin.updateRankings({}, function success(data) {
	        $scope.reset();
	        $scope.pageLoading--;

	        if (data.Added.length > 0) {
	            var message = data.Added.map(function (obj) {
	                return obj;
	            }).join('<br>');

	            ons.notification.alert({
	                messageHTML: message,
	                title: 'Athletes Added:',
	                buttonLabel: 'OK',
	                animation: 'default',
	                callback: function () {
	                    // Alert button is closed!
	                }
	            });
	        }

	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.handleError(err.data ? err.data.Message : 'Error Updating Rankings');
	    });
	};

	$scope.addSocial = function () {
	    $scope.editingAthlete.social.push({ });
	};
	$scope.removeSocial = function (item) {
	    var index = $scope.editingAthlete.social.indexOf(item);
	    $scope.editingAthlete.social.splice(index, 1);
	};
	$scope.addEquipment = function () {
	    $scope.editingAthlete.equipment.push({ });
	};
	$scope.removeEquipment = function (item) {
	    var index = $scope.editingAthlete.equipment.indexOf(item);
	    $scope.editingAthlete.equipment.splice(index, 1);
	};
}]);