﻿module.controller('AdminChallengeCtrl', ['$scope', '$routeParams', '$log', '$window', '$translate', 'dataService', 'UploadService', function ($scope, $routeParams, $log, $window, $translate, dataService, UploadService) {
	$scope.challenges = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingChallenge = {};
	$scope.error = {};
	$scope.searchName = '';

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'challenges' && !$scope.loaded) {
				loadChallenges();
			}
		});

	function loadChallenges() {
		$scope.pageLoading++;
		dataService.Challenge.get(function success(challenges) {
			angular.forEach(challenges, function (value, key) {
				value.active_date = new Date(value.active_date.substring(0, 10));
				value.end_date = new Date(value.end_date.substring(0, 10));
			});
			$scope.challenges = challenges;
			$scope.loaded = true;
			$scope.pageLoading--;
		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Challenges';
		});
	};

	function loadDisciplines() {
		$scope.pageLoading++;
		dataService.Admin.disciplines(function success(disciplines) {
			$scope.disciplines = disciplines;
			$scope.loaded = true;
			$scope.pageLoading--;
		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Disciplines';
		});
	};

	function loadRaces() {
		$scope.pageLoading++;
		dataService.Race.query(function success(races) {
			$scope.races = races;
			$scope.loaded = true;
			$scope.pageLoading--;
		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Races';
		});
	};

	//----UI functions----
	$scope.isEditing = function (id) {
		return id == $scope.editingChallenge.id;
	};

	$scope.edit = function (challenge) {
		if ($scope.isEditing(challenge.id)) {
			// if I'm already editing the challenge and I click on 'edit' again, then clear it and close
			$scope.reset();
		}
		else {
			// only set the edit if I'm not already editing it (or I'm editing a different challenge)
			$scope.reset();
			$scope.editingChallenge = angular.copy(challenge);
			$scope.imageFile = null;

			//only load races if adding a new challenge
			if (challenge.id == 0) {
				loadDisciplines();
				loadRaces();
				$scope.editingChallenge.parameters = { disciplines: [] };
			}
		}
	};

	$scope.delete = function (challenge) {
		var deleteChallenge = $window.confirm('Are you sure you want to delete this challenge?');

		if (deleteChallenge) {
			$scope.pageLoading++;
			dataService.Challenge.delete({ id: challenge.id }, function success() {
				var index = $scope.challenges.indexOf(challenge);
				$scope.challenges.splice(index, 1);

				$scope.reset();
				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Deleting Challenge';
			});
		}
	};

	$scope.reset = function () {
		$scope.editingChallenge = {};
	};

	//listen for the file selected event
	$scope.$on("fileSelected", function (event, args) {
		$scope.$apply(function () {
			$scope.imageFile = args.file;
		});

		// update the image for preview
		var reader = new FileReader();
		reader.onload = function (e) {
			$scope.$apply(function () {
				$scope.editingChallenge.preview_path = e.target.result;
			});
		}
		reader.readAsDataURL(args.file);
	});

	$scope.save = function (challenge) {
		$scope.pageLoading++;

		if (challenge.id == 0) { //if adding a challenge, build out the whole object, with parameters, disciplines, races, etc.
			if (challenge.hasGoal) {
				challenge.challengegoal = { challenge_id: challenge.id, goal_type: challenge.goal.type, goal: challenge.goal.score }
			}
			if (challenge.filterType == "parameters") {
				params = challenge.parameters;
				challenge.challengeparameter = { challenge_id: challenge.id, start_date: params.start_date, end_date: params.end_date, location: params.location };
				challenge.challengeparameterdisciplines = [];
				angular.forEach(params.disciplines, function (value, key) {
					challenge.challengeparameterdisciplines.push({ challenge_id: challenge.id, discipline_id: value })
				});
			}
			if (challenge.filterType == "races") {
				challenge.challengeraces = [];
				angular.forEach(challenge.races.racelist, function (value, key) {
					challenge.challengeraces.push({ challenge_id: challenge.id, race_id: value })
				});
			}
		}

		dataService.Challenge.save(challenge, function success(savedChallenge) {
			//now save the image
			if ($scope.imageFile) {
				UploadService.saveChallengeImage(savedChallenge.id, $scope.imageFile).then(
					function (data) {
						savedChallenge.image_path = data.image_path;
					},
					function (message) {
						alert(message);
					});
			}

			//update the local list
			if (challenge.id == 0) {
				$scope.challenges.push(savedChallenge);
			}
			else {
				angular.forEach($scope.challenges, function (value, key) {
					if (value.id == savedChallenge.id) {
						savedChallenge.active_date = new Date(savedChallenge.active_date.substring(0, 10));
						savedChallenge.end_date = new Date(savedChallenge.end_date.substring(0, 10));
						$scope.challenges[key] = savedChallenge;
					}
				});
			}
			
			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Challenge';
		});
	};
}]);