﻿module.controller('AdminClubCtrl', ['$scope', '$routeParams', '$log', 'dataService', 'UploadService', function ($scope, $routeParams, $log, dataService, UploadService) {
	$scope.clubs = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingClub = {};
	$scope.error = {};
	$scope.searchName = '';

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'clubs' && !$scope.loaded) {
				loadClubs();
			}
		});

	function loadClubs() {
		$scope.pageLoading++;
		dataService.Club.query(function success(clubs) {
			$scope.clubs = clubs;
			$scope.loaded = true;
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Clubs';
		});
	};

	//----UI functions----
	$scope.isEditing = function (id) {
		return id == $scope.editingClub.id;
	};

	$scope.edit = function (club) {
		if ($scope.isEditing(club.id)) {
			// if I'm already editing the club and I click on 'edit' again, then clear it and close
			$scope.reset();
		}
		else {
			// only set the edit if I'm not already editing it (or I'm editing a different club)
			$scope.reset();
			$scope.editingClub = angular.copy(club);
			$scope.imageFile = null;
		}
	};

	$scope.delete = function (club) {
		$scope.pageLoading++;
		dataService.Club.delete({ id: club.id }, function success() {
			var index = $scope.clubs.indexOf(club);
			$scope.clubs.splice(index, 1);

			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Deleting Club';
		});
	};

	$scope.reset = function () {
		$scope.editingClub = {};
	};

	//listen for the file selected event
	$scope.$on("fileSelected", function (event, args) {
		$scope.$apply(function () {
			$scope.imageFile = args.file;
		});

		// update the image for preview
		var reader = new FileReader();
		reader.onload = function (e) {
			$scope.$apply(function () {
				$scope.editingClub.preview_path = e.target.result;
			});
		}
		reader.readAsDataURL(args.file);
	});

	$scope.save = function (club) {
		$scope.pageLoading++;
		dataService.Club.save(club, function success(savedClub) {
			//now save the image
			if ($scope.imageFile) {
				UploadService.saveClubImage(club.id, $scope.imageFile).then(
					function (data) {
						savedClub.image_path = data.image_path;
					},
					function (message) {
						alert(message);
					});
			}

			//update the local list
			if (club.id == 0) {
				$scope.clubs.push(savedClub);
			}
			else {
				angular.forEach($scope.clubs, function (value, key) {
					if (value.id == savedClub.id) {
						$scope.clubs[key] = savedClub;
					}
				});
			}
			
			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Club';
		});
	};
}]);