﻿module.controller('AdminRaceCtrl', ['$scope', '$routeParams', '$log', '$translate', 'dataService', function ($scope, $routeParams, $log, $translate, dataService) {
	$scope.races = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingRace = {};
	$scope.resultingRaceId = 0;
	$scope.error = {};
	$scope.select = { disciplines: [] };

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'races' && !$scope.loaded) {
			    loadRaces();
			    loadDisciplines();
			}
		});

	function loadDisciplines() {
	    $scope.pageLoading++;
	    dataService.Admin.disciplines(function success(disciplines) {
	        $scope.select.disciplines = disciplines.map(function(item) {
	            return { id: item, text: $translate.instant(item) }
	        });
	        $scope.pageLoading--;
	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Disciplines';
	    });
	};

	function loadRaces() {
		$scope.pageLoading++;
		dataService.Race.query(function success(races) {
			angular.forEach(races, function (value, key) {
				value.race_date = new Date(value.race_date.substring(0, 10));
				value.race_time = new Date(value.race_time);
			});
			$scope.races = races;
			$scope.loaded = true;
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Races';
		});
	};

	//----UI functions----
	$scope.isEditing = function (id) {
		return id == $scope.editingRace.id;
	};

	$scope.isResulting = function (id) {
		return id == $scope.resultingRaceId;
	};

	$scope.edit = function (race) {
		$scope.reset();
		$scope.editingRace = angular.copy(race);
	};

	$scope.delete = function (race) {
		$scope.pageLoading++;
		dataService.Race.delete({ id: race.id }, function success() {
			var index = $scope.races.indexOf(race);
			$scope.races.splice(index, 1);

			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Deleting Race';
		});
	};

	$scope.uploadResults = function (id) {
		$scope.reset();
		$scope.resultingRaceId = id;
	};

	$scope.uploadResultsConfirm = function (csv) {
		var json = { race_id: $scope.resultingRaceId, csv: csv };

		$scope.pageLoading++;
		dataService.Admin.uploadRaceResults(json, function success(data) {
			$scope.reset();
			$scope.pageLoading--;

			if (data.Added.length > 0) {
				var message = data.Added.map(function (obj) {
					return obj;
				}).join('<br>');

				ons.notification.alert({
					messageHTML: message,
					title: 'Athletes Added:',
					buttonLabel: 'OK',
					animation: 'default', 
					callback: function () {
						// Alert button is closed!
					}
				});
			}

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Processing Results';
		});
	};

	$scope.reset = function () {
		$scope.editingRace = {};
		$scope.resultingRaceId = 0;
	};

	$scope.processResults = function (id) {
	    $scope.reset();
	    $scope.resultingRaceId = id;
	};

	$scope.processResultsConfirm = function () {
	    $scope.pageLoading++;
	    dataService.Admin.processRaceResults({ id: $scope.resultingRaceId }, function success(data) {
	        angular.forEach($scope.races, function (value, key) {
	            if (value.id == $scope.resultingRaceId) {
	                $scope.races[key].completed = true;
	            }
	        });

	        $scope.reset();
	        $scope.pageLoading--;
	        ons.notification.alert({
	            messageHTML: 'Processing complete!',
	            title: 'Race processing complete',
	            buttonLabel: 'OK',
	            animation: 'default',
	            callback: function () {
	                // Alert button is closed!
	            }
	        });

	    }, function error(err) {
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Processing Results';
	    });
	};

	$scope.save = function (race) {
		$scope.pageLoading++;
		dataService.Race.save(race, function success(savedRace) {
		    savedRace.race_date = new Date(savedRace.race_date.substring(0, 10));
		    savedRace.race_time = new Date(savedRace.race_time);

			if (race.id == 0) {
				$scope.races.push(savedRace);
			}
			else {
				angular.forEach($scope.races, function (value, key) {
					if (value.id == savedRace.id) {
						$scope.races[key] = savedRace;
					}
				});
			}
			
			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Race';
		});
	};
}]);