﻿module.controller('AdminTourCtrl', ['$scope', '$routeParams', '$log', '$window', 'dataService', 'UploadService',
	function ($scope, $routeParams, $log, $window, dataService, UploadService) {
	$scope.tours = [];
	$scope.loaded = false;
	$scope.pageLoading = 0;
	$scope.editingTour = {};
	$scope.error = {};

	//----Load functions----
	$scope.$watch(
		function (scope) {
			return scope.activeTab;
		},
		function (newValue, oldValue) {
			if (newValue == 'tours' && !$scope.loaded) {
				loadTours();
			}
		});

	function loadTours() {
		$scope.pageLoading++;
		dataService.Tour.query(function success(tours) {
			$scope.tours = tours;
			$scope.loaded = true;
			$scope.pageLoading--;
		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Tours';
		});
	};

	//----UI functions----
	$scope.isEditing = function (id) {
		return id == $scope.editingTour.id;
	};

	$scope.edit = function (tour) {
		if ($scope.isEditing(tour.id)) {
			// if I'm already editing the tour and I click on 'edit' again, then clear it and close
			$scope.reset();
		}
		else {
			// only set the edit if I'm not already editing it (or I'm editing a different ad)
			$scope.reset();
			$scope.editingTour = angular.copy(tour);
			if ($scope.editingTour.id == 0) {
				$scope.editingTour.active = true;
			}
		}
	};

	$scope.save = function (tour) {
		if (tour.id == '') {
			// for a new ad, pre-populate some fields that will get set later, on image upload
		}

		$scope.pageLoading++;
		dataService.Tour.save(tour, function success(savedTour) {
			// now save the image

			if (tour.id == '') { // add new mode
				$scope.tours.push(savedTour);
			}
			else {  // edit existing mode
				angular.forEach($scope.tours, function (value, key) {
				    if (value.id == savedTour.id) {
				        $scope.tours[key] = savedTour;
					}
				});
			}

			$scope.reset();
			$scope.pageLoading--;

		}, function error(err) {
			$scope.pageLoading--;
			$log.debug('Err:' + JSON.stringify(err));
			$scope.error.message = err.data ? err.data.Message : 'Error Saving Tour';
		});
	}

	$scope.delete = function (tour) {
		var deleteTour = $window.confirm('Are you sure you want to delete this tour?');

		if (deleteTour) {
			$scope.pageLoading++;
			dataService.Tour.delete({ id: tour.id }, function success() {
				var index = $scope.tours.indexOf(tour);
				$scope.tours.splice(index, 1);

				$scope.reset();
				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Deleting Tour';
			});
		}
	};


	$scope.reset = function () {
		$scope.editingTour = { };
	};

}]);