﻿module.controller('AthleteHistoryCtrl', ['$rootScope', '$scope', '$log', '$translate', 'dataService', 'navService', function ($rootScope, $scope, $log, $translate, dataService, navService) {

    var vm = this;
    vm.loading = { value: 0 };
    vm.error = { message: '' };

    var page = $rootScope.getNavigator().getCurrentPage();
    var athleteId = page.options.athleteId; 

    vm.filterOptions = {
        seasonOptions: [
            { id: '2016', text: '2016' },
            { id: '2017', text: '2017' },
            { id: 'All', text: $translate.instant('All'), selected: true }
        ],
        locationOptions: [
            { id: 'All', text: $translate.instant('All Locations') }
        ],
        disciplineOptions: []
    };

    vm.historyFilter = {
        season: 'All',
        location: 'All',
        discipline: 'All'
    };

    function getDisciplines() {
        if (vm.filterOptions.disciplineOptions.length <= 1) {
            vm.loading.value++;
            dataService.Admin.disciplines(function success(data) {
                data.forEach(function (item) {
                    vm.filterOptions.disciplineOptions.push({ id: item, text: item });
                });
                // push 'All' option onto end
                vm.filterOptions.disciplineOptions.push({ id: 'All', text: $translate.instant('All'), selected: true });
                vm.loading.value--;
            }, function error(err) {
                vm.loading.value--;
                $log.debug('Err:' + JSON.stringify(err));
                $scope.error.message = err.data ? err.data.Message : 'Error retrieving disciplines';
            });
        }
    };

    function getLocations() {
        if (vm.filterOptions.locationOptions.length <= 1) {
            vm.loading.value++;
            dataService.Athlete.getLocations({ id: athleteId },
                function success(data) {
                    vm.loading.value--;
                    data.forEach(function (item) {
                        vm.filterOptions.locationOptions.push({ id: item, text: item });
                    });
                }, function error(err) {
                    vm.loading.value--;
                    $log.debug('Error:' + JSON.stringify(err));
                    vm.error.message = err.data ? err.data.Message : 'Error retrieving locations';
                });
        }
    };

    function getAthleteHistory() {
        vm.loading.value++;
        var requestData = {
            athletes: [athleteId],
            filters: [
                { name: 'startSeason', value: vm.historyFilter.season == "All" ? '2016' : vm.historyFilter.season },
                { name: 'endSeason', value: vm.historyFilter.season == "All" ? '2017' : vm.historyFilter.season },
                { name: 'location', value: vm.historyFilter.location },
                { name: 'discipline', value: vm.historyFilter.discipline }
            ]
        };
        dataService.Athlete.history(requestData,
            function success(data) {
                vm.loading.value--;
                vm.history = data;
                vm.athlete = vm.history.athletes[0].athlete;
                vm.races = vm.history.athletes[0].races;
                vm.results = { maxPlacement: data.maxPlacement, races: vm.races };
            }, function error(err) {
                vm.loading.value--;
                $log.debug('Error:' + JSON.stringify(err));
                vm.error.message = err.data ? err.data.Message : 'Error retrieving athlete history';
            });
    };

    $scope.$watch(
		function (scope) {
		    return scope.athleteHistory.historyFilter.season;
		},
		function (newValue, oldValue) {
		    if (newValue && oldValue && newValue !== oldValue) {
		        vm.loadAll();		        
		    }
		});

    $scope.$watch(
        function (scope) {
            return scope.athleteHistory.historyFilter.discipline;
        },
        function (newValue, oldValue) {
            if (newValue && oldValue && newValue !== oldValue) {
                vm.loadAll();
            }
        });

    vm.loadAll = function () {
        getAthleteHistory();
        getDisciplines();
        getLocations();
    };

    function init() {
        vm.loadAll();
    }

    init();

}]);