module.controller('ChallengesDetailClubCtrl', ['$log', '$rootScope', '$scope', '$location', '$routeParams', '$translate', '$filter', '$sce', 'dataService', 'socialService', 'navService', function ($log, $rootScope, $scope, $location, $routeParams, $translate, $filter, $sce, dataService, socialService, navService) {
	$scope.error = {
		message: ""
	};
	$scope.overallLeaders = [];
	$scope.followingLeaders = [];

	$scope.leaderOptions = [{ id: 'overall', text: $translate.instant('Competing Clubs'), selected: true }, { id: 'following', text: $translate.instant('My USSA Club') }];
	$scope.leaderSelected = 'overall';

	$scope.pageLoading = 0;
	var page = $rootScope.getNavigator().getCurrentPage();
	$scope.challenge = page.options.challenge;

	navService.push({ title: $translate.instant($scope.challenge.name), controlContext: false });

	$scope.challenge.htmlDescription = $sce.trustAsHtml($scope.challenge.long_description);

	loadClubs();

    //---API functions----
	function loadClubs() {
		if ($scope.overallLeaders.length === 0) {
			$scope.pageLoading++;
			$scope.overallLeaders = [];
			dataService.Challenge.competingClubs({ id: $scope.challenge.id }, function success(data) {
				$scope.overallLeaders = $filter('orderBy')(data, ['club.rank', 'club.name']);

				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Followers';
			});
		}
	}

	function loadMyClub() {
		if ($scope.followingLeaders.length === 0) {
			$scope.pageLoading++;
			$scope.selectedLeaders = [];
			dataService.Challenge.myClub({ id: $scope.challenge.id, clubid: $scope.challenge.club_id }, function success(data) {
			    $scope.followingLeaders = $filter('orderBy')(data.leaders, 'user.rank');
			    for (var i = 0; i < data.score_contributor_count; i++) {
			        if ($scope.followingLeaders[i] && $scope.followingLeaders[i].user) {
			            $scope.followingLeaders[i].user.highlight = true;
			        }
			    }
				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Followers';
			});
		}
	}


    //---UI functions----
	$scope.backToChallenges = function () {
	    $rootScope.popPage();
	    $location.search({});
	};

	$scope.showRaces = function () {
	    $rootScope.pushPage('partials/challenges/races.html', { challenge: $scope.challenge });
	    navService.push({ title: $translate.instant('Races'), controlContext: false });
	};

	$scope.isJoined = function () {
		return $scope.challenge.joined;
	};

	//---WATCH functions----
	function refreshLeaders() {
		if ($scope.leaderSelected === "overall") {
			loadClubs();
		}
		else {
			loadMyClub();
		}
	}
	$scope.$watch(
		function (scope) {
			return scope.leaderSelected;
		},
		function (newValue, oldValue) {
			if (newValue !== oldValue) {
				refreshLeaders();
			}
		});
}]);

