module.controller('ChallengesDetailCtrl', ['$log', '$rootScope', '$scope', '$location', '$routeParams', '$translate', '$filter', '$sce', 'TourService', 'LocalNotificationService', 'dataService', 'socialService', 'navService',
	function ($log, $rootScope, $scope, $location, $routeParams, $translate, $filter, $sce, TourService, LocalNotificationService, dataService, socialService, navService) {
	$scope.error = {
		message: ""
	};
	$scope.overallLeaders = [];
	$scope.followingLeaders = [];

	$scope.leaderOptions = [{ id: 'overall', text: $translate.instant('Overall'), selected: true }, { id: 'following', text: $translate.instant('Following') }];
	$scope.leaderSelected = 'overall';

	$scope.pageLoading = 0;
	var page = $rootScope.getNavigator().getCurrentPage();
	$scope.challenge = page.options.challenge;
	$scope.challenge.htmlDescription = $sce.trustAsHtml($scope.challenge.long_description);

	loadOverall();

    //---API functions----
	function loadOverall() {
		if ($scope.overallLeaders.length == 0) {
			$scope.pageLoading++;
			$scope.selectedLeaders = [];
			dataService.Challenge.overall({ id: $scope.challenge.id }, function success(data) {
				orderOverallLeaders(data);

				$scope.selectedLeaders = $scope.overallLeaders;

				$scope.pageLoading--;
				TourService.Start("challengeDetails");

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Followers';
			});
		}
		else {
			orderOverallLeaders($scope.overallLeaders);
			$scope.selectedLeaders = $scope.overallLeaders;
		}
	}

	function loadFollowing() {
		if ($scope.followingLeaders.length <= 1) {
			$scope.pageLoading++;
			$scope.selectedLeaders = [];
			dataService.Challenge.following({ id: $scope.challenge.id }, function success(data) {
				orderFollowingLeaders(data);

				$scope.selectedLeaders = $scope.followingLeaders;

				$scope.pageLoading--;

			}, function error(err) {
				$scope.pageLoading--;
				$log.debug('Err:' + JSON.stringify(err));
				$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Followers';
			});
		}
		else {
			orderFollowingLeaders($scope.followingLeaders);
			$scope.selectedLeaders = $scope.followingLeaders;
		}
	}

	function orderFollowingLeaders(data) {
		$scope.followingLeaders = $filter('orderBy')(data, 'user.rank');
	}

	function orderOverallLeaders(data) {
		$scope.overallLeaders = $filter('orderBy')(data, 'user.rank');
	}


    //---UI functions----
	$scope.backToChallenges = function () {
	    $rootScope.popPage();
	    $location.search({});
	};

	$scope.showRaces = function () {
	    $rootScope.pushPage('partials/challenges/races.html', { challenge: $scope.challenge });
	    navService.push({ title: $translate.instant('Races'), controlContext: false });
	};

	$scope.isJoined = function () {
		return $scope.challenge.joined;
	};

	$scope.join = function() {
	    $scope.pageLoading++;
	    $scope.challenge.loading = true;
	    dataService.Challenge.join({ id: $scope.challenge.id }, function success(data) {
	    	//add new user that's in the top 10 to leaders list
	    	if (data.user.rank <= 10) {
	    		$scope.overallLeaders.push({ score: data.score, user: data.user });

	    		//bump all user ranks that have a worse score than me
	    		angular.forEach($scope.overallLeaders, function (value, key) {
	    			if (value.score < data.score) {
	    				value.user.rank += 1;
	    			}
	    		});
	    	}

	    	//add new user to following list
	    	$scope.followingLeaders.push(data);
	    	//bump all user ranks that have a worse score than me
	    	angular.forEach($scope.followingLeaders, function (value, key) {
	    		if (value.score < data.score) {
	    			value.user.rank += 1;
	    		}
	    	});

	    	refreshLeaders();
	        $scope.challenge.joined = true;
	        $scope.challenge.loading = false;

	        $scope.challenge.participantcount = $scope.challenge.participantcount + 1;
	        $scope.challenge.rank = data.user.rank;
	        $scope.challenge.score = data.score;

	        LocalNotificationService.UpdateChallengeNotifications([$scope.challenge]);

	        $scope.pageLoading--;

	    }, function error(err) {
	        $scope.challenge.loading = false;
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Joining Challenge';
	    });
	};

	$scope.leave = function () {
	    $scope.pageLoading++;
	    $scope.challenge.loading = true;
	    dataService.Challenge.leave({ id: $scope.challenge.id }, function success(data) {
	        $scope.challenge.joined = false;
	        $scope.challenge.loading = false;
	        $scope.challenge.participantcount = $scope.challenge.participantcount - 1;

	        LocalNotificationService.UpdateChallengeNotifications([$scope.challenge]);

	        $scope.pageLoading--;

	    }, function error(err) {
	        $scope.challenge.loading = false;
	        $scope.pageLoading--;
	        $log.debug('Err:' + JSON.stringify(err));
	        $scope.error.message = err.data ? err.data.Message : 'Error Leaving Challenge';
	    });
	};

	//---WATCH functions----
	function refreshLeaders() {
		if ($scope.leaderSelected == "overall") {
			loadOverall();
		}
		else {
			loadFollowing();
		}
	}
	$scope.$watch(
		function (scope) {
			return scope.leaderSelected;
		},
		function (newValue, oldValue) {
			if (newValue != oldValue) {
				refreshLeaders();
			}
		});
}]);

