﻿module.controller('CounterCtrl', ['$scope', '$interval', '$routeParams', 'dataService', function ($scope, $interval, $routeParams, dataService) {
    var usersPerPage = 45;
    var newUsersPerStep = 3;
    var donationPerUser = 50;
    var donationPerPickUser = 25;
    var donationMax = 20000;
    var startDate = $routeParams.startDate || '2016-01-01';
    var raceId = $routeParams.raceId || 374;

    $scope.raceId = raceId;
	$scope.startDate = new Date(startDate).getTime();
	$scope.sinceDate = new Date(startDate + 'T00:00:00').getTime();
	$scope.totalCount = 0;
	$scope.totalAmount = 0;
	$scope.allUsers = [];
	$scope.users = [];
	$scope.currentUser = -1;
	$scope.currentStep = -1;
	$scope.steps = ['1', '2', '3', '4'];
	$scope.step = $scope.steps[$scope.currentStep];

	function changeStep() {
	    $scope.currentStep = $scope.currentStep + 1;
	    if ($scope.currentStep >= $scope.steps.length || $scope.currentStep < 0) {
	        $scope.currentStep = 0;
	    }
	    $scope.step = $scope.steps[$scope.currentStep];
	}

	function loadHighlightedUsers() {
	    if ($scope.allUsers.length > 0) {
			var newUsers = [];
			for (i = 0; i < newUsersPerStep; i++) {
			    $scope.currentUser++;
			    if ($scope.currentUser >= $scope.allUsers.length) {
			        $scope.currentUser = $scope.allUsers.length - 1;
			        break;
			    }
				newUsers.push($scope.allUsers[$scope.currentUser]);
			}
			newUsers = newUsers.concat($scope.users);
			$scope.users = newUsers.slice(0, usersPerPage);
		}
	}

	function loadTotals() {
		dataService.Users.getTotals({}, { sinceDate: $scope.startDate }, function success(data) {
			$scope.totalCount = data.Count;
			$scope.totalAmount = Math.min(data.Count * donationPerUser, donationMax);
		}, function error(err) {
			
		});
	};

	function loadLatestUsers() {
	    dataService.Users.getLatest({}, { sinceDate: $scope.sinceDate }, function success(data) {
			if (data.length) {
				$scope.sinceDate = new Date(data[0].created).getTime();
				data.forEach(function (user) {
					$scope.allUsers.push(user);
				});
				if ($scope.currentUser == -1) {
					loadHighlightedUsers();
				}
			}
		}, function error(err) {

		});
	};

	function loadPickTotals() {
	    dataService.Users.getPickTotals({}, { raceId: $scope.raceId }, function success(data) {
	        $scope.totalCount = data.Count;
	        $scope.totalAmount = Math.min(data.Count * donationPerPickUser, donationMax);
	    }, function error(err) {

	    });
	};

	function loadPickLatestUsers() {
	    if ($scope.latestResolved) {
	        $scope.latestResolved = false;
	        dataService.Users.getPickLatest({}, { raceId: $scope.raceId, sinceDate: $scope.sinceDate }, function success(data) {
	            $scope.latestResolved = true;
	            if (data.length) {
	                $scope.sinceDate = new Date(data[0].created).getTime();
	                data.forEach(function (user) {
	                    $scope.allUsers.push(user);
	                });
	                if ($scope.currentUser == -1) {
	                    loadHighlightedUsers();
	                }
	            }
	        }, function error(err) {

	        });
	    }
	};

	var refresh_old = function () {
	    changeStep();
	    loadTotals();
	    loadLatestUsers();
	};

	var refresh = function () {
	    changeStep();
	    loadPickTotals();
	    loadPickLatestUsers();
	};

	$scope.latestResolved = true;
	refresh();

	$interval(refresh, 15000);
	$interval(loadHighlightedUsers, 2000 * newUsersPerStep);
}]);