module.controller('LeagueInviteCtrl',
    ['$rootScope', '$scope', '$routeParams', '$storage', '$location', 'AuthService','TourService',
        'AccountService', 'TranslationService', 'dataService',
        function ($rootScope, $scope, $routeParams, $storage, $location, AuthService, TourService, AccountService, TranslationService, dataService) {

            var THROTTLE = 500;
            $scope.error = {
                message: ""
            };
            $scope.searching = 0;

            //parameter passing from last page
            var page = $rootScope.getNavigator().getCurrentPage(),
                isNewLeague = page.options.isNewLeague,
                leagueId = page.options.leagueId;

            TourService.Start("leagueInvite");

            //get the users id
            var currentUser = $storage.get('userData');
            $scope.currentUser = currentUser;

            //---UI Collections---
            $scope.userSearchResults = [];
            $scope.invitees = [];

            //---IO functions----

            var searchingUsers = 0;
            $scope.searchUsers = searchUsers;

            var lastSearch;
            var nextSearchFn;

            function searchUsers() {
                if (lastSearch &&  (new Date() - lastSearch < THROTTLE)) {
                    //if there is already a search, cancel it
                    if (nextSearchFn)clearTimeout(nextSearchFn);
                    nextSearchFn = setTimeout(function () {
                        doSearch();
                    },THROTTLE);

                }
                else{

                    doSearch();
                }
            }

            function doSearch(){
                var value = $scope.txtUserSearch;

                //Only search after 2 chars
                if(value.length<2){
                    return;
                }

                $scope.searching = 1;
                dataService.Users.search({search:value.toLowerCase()},{},function success(data){
                    //mark if already invited
                    data.forEach(function(user){

                        if($scope.invitees.map(function(u){return u.id}).indexOf(user.id)>-1){
                            user.isInvited = true;
                        }
                    })
                    $scope.userSearchResults = _.filter(data,function(item){
                        if(item.id == currentUser.id){
                            return false;//Dont add self
                        }
                        return true;
                    });
                    $scope.searching = 0;
                }, function error(err) {
                	$scope.searching = 0;
                    $scope.error.message = err.data.Message;
                })
                lastSearch = new Date();
            }

            var emailOnlyInvites = 0;
            $scope.addToInvitees = addToInvitees;
            function addToInvitees(user) {
				if (user.email) user.email = user.email.toLowerCase();
                if (!user.isInvited
                    //Check to see if the user id has already been added
                    && $scope.invitees.map(function (u) {
                    return u.id
                }).indexOf(user.id) == -1
                    //Check to see if the email has already been added
                    && (user.email ? $scope.invitees.map(function (u) {
                        return u.email.toLowerCase()
                    }).indexOf(user.email.toLowerCase()) == -1 : false)) {

                    if (!user.id) {
                        emailOnlyInvites++;
                        user.id = -1 * emailOnlyInvites;
                    }
                    user.isInvited = true;
                    $scope.invitees.push(user);

                    $scope.nonMemberEmail = '';
                }
            }

            $scope.removeFromInvitees = removeFromInvitees;
            function removeFromInvitees(user) {
                var index = $scope.invitees.map(function (u) {
                    return u.id
                }).indexOf(user.id);
                if (user.isInvited && index != -1) {
                    user.isInvited = false;
                    if (index > -1) {
                        $scope.invitees.splice(index, 1);
                    }
                }

                var sIndex = $scope.userSearchResults.map(function (u) {
                    return u.id
                }).indexOf(user.id);
                if (sIndex != -1) {
                    $scope.userSearchResults[sIndex].isInvited = false;
                }
            }

            $scope.sendInvites = sendInvites;
            function sendInvites() {
                var unsent = $scope.invitees.filter(function (invite) {
                    return !invite.isPending;
                });
                if (unsent.length) {
                    $rootScope.loading++;
                    dataService.Invites.sendInvites({}, { leagueId: leagueId, invites: unsent }, function success(data) {
                        $rootScope.loading--;
                        $scope.tabbar.loadPage('partials/leagues/index.html');
                    }, function error(err) {
                        $rootScope.loading--;
                        $scope.error.message = "Error sending invites.";
                    })
                }
                else {
                    $scope.error.message = "No new invites to send.";
                    return;
                }
            }
        }]);
