module.controller('LeaguesCtrl', ['$rootScope', '$scope', '$timeout', '$routeParams', '$location', '$storage', '$translate', 'AuthService', 'AccountService', 'TranslationService', 'TourService', 'AppStateService', 'dataService', 'navService',
    function ($rootScope, $scope, $timeout, $routeParams, $location, $storage, $translate, AuthService, AccountService, TranslationService, TourService, AppStateService, dataService, navService) {
        $scope.error = {
            message: ""
        };
        $scope.pageLoading = 0;

    	//Page states enum
        var state = { LOCKED: 0, EDIT: 1 };
        $scope.state = state.LOCKED;

        //get the leagues
        var page = $rootScope.getNavigator().getCurrentPage(),
            raceId = page.options.raceId;

        navService.initializeStack({ title: $translate.instant('Leagues'), controlContext: true });

        //get the users id
        var currentUser = $storage.get('userData');
        $scope.currentUser = currentUser;
        //---Scope Properties
        $scope.selectedLeague = { id: AppStateService.getTabContext() || AppStateService.getLeagueId()};
        $scope.ranks_closed = 0;
        $scope.matchups_closed = 0;
        $scope.invites_closed = 0;
        $scope.selectedWeek;
        $scope.minWeek = 1;
        $scope.maxWeek = 17;
        $scope.minPlayers = 6;
        $scope.maxPlayers = 12;

        $scope.scoreDetails;
        //---UI Collections---
        $scope.invites = [];
        $scope.pendingInvites = [];

        //--Initial Data Load
        loadLeagues();
        loadInvites();

        //---IO functions----
        function loadExistingInvites(league) {
            if (!isCaptain())return;
            dataService.Invites.getPendingInvites({ leagueId: league.id }, function success(data) {
                //load existing
                $scope.pendingInvites = data;
            }, function (err) {

            });
        }

        function loadLeagues(noLoading) {
            if (!noLoading) $scope.pageLoading++;
            dataService.Leagues.query(function success(data) {
                if (!noLoading) $scope.pageLoading--;
                $scope.userLeagues = data;
                if (data.length) {
                    var index = 0;
                    if ($scope.selectedLeague && $scope.selectedLeague.id) {
                        selectedIndex = $scope.userLeagues.map(function (u) {
                            return u.id;
                        }).indexOf($scope.selectedLeague.id);
                        if (selectedIndex >= 0) {
                            index = selectedIndex;
                        }
                    }
                    $scope.selectedLeague = $scope.userLeagues[index];
                    TourService.Start("leagues");
                }
            }, function error(err) {
                if (!noLoading) $scope.pageLoading--;

            });
        }

        function loadInvites() {
            if (!currentUser)return;
            $scope.pageLoading++;
            dataService.Invites.get({}, currentUser, function success(data) {
                $scope.pageLoading--;
                $scope.invites = data;
            }, function error(err) {
                $scope.pageLoading--;
            });
        }

        //---UI functions----
        $scope.unloadScoreDetails = function () {
            $scope.showDetails = false;
        };
        $scope.loadScoreDetails = function ($event, matchup) {
            $scope.showDetails = true;
            $scope.scoreDetailsLoaded = false;
            if ($event.view.innerWidth > 800) {
                $scope.showDetailsStyle = { top: (($event.view.innerHeight * 0.30) + $event.target.closest('div.page__content').scrollTop) + "px" };
            }
            else {
                $scope.showDetailsStyle = { top: (10 + $event.target.closest('div.page__content').scrollTop) + "px" };
            }
            //$timeout(function () {
                dataService.UserRaceResults.compare({ week: matchup.week, user1: matchup.team1.user_id, user2: matchup.team2.user_id }, function success(data) {
                    $scope.scoreDetailsLoaded = true;
                    // check data length -- if it has more than the two default properties ($promise, $resolved), set score details
                    $scope.scoreDetails = Object.keys(data).length > 2 ? data: undefined;
                }, function error(err) {

                });
            //}, 10);
        };

        $scope.toggleEditState = toggleEditState;
        function toggleEditState() {
        	$scope.state = Math.abs($scope.state - 1);
        }

        $scope.isEditable = isEditable;
        function isEditable(userRank) {
        	if ($scope.state !== state.EDIT) return false;
        	if (userRank.team_id && !userRank.isUser) return true;
        	return false;
        }

        $scope.removePlayer = removePlayer;
        function removePlayer(user) {
            user.loading = 1;
            dataService.Teams.remove({leagueId: $scope.selectedLeague.id, teamId: user.team_id},function success(data){
            	//remove the user from the list
            	var index = $scope.selectedLeague.team.map(function (u) {return u.team_id;}).indexOf(user.team_id);
            	if (index > -1) {
            		$scope.selectedLeague.team.splice(index, 1);
            	}
            },function(err){

                user.loading = 0;
            })

        }

        $scope.roomForPlayers = roomForPlayers;
        function roomForPlayers() {
        	if (!$scope.selectedLeague) return false;
            var currentPlayers = ($scope.invites.length + $scope.selectedLeague.team.length);
            return currentPlayers < $scope.maxPlayers;
        }

        $scope.needMorePlayers = needMorePlayers;
        function needMorePlayers() {
            if (!$scope.selectedLeague || !$scope.selectedLeague.team) return false;
        	var currentPlayers = ($scope.selectedLeague.team.length);
        	return currentPlayers < $scope.minPlayers;
        }

        $scope.isCaptain = isCaptain;
        function isCaptain(user) {
            if (!$scope.selectedLeague) return false;
            return $scope.selectedLeague.captain_id === currentUser.id;
        }

        $scope.changeWeek = changeWeek;
        function changeWeek(size) {
            if (!size) {
                $scope.selectedWeek = assembleWeek(getCurrentWeek());
            }
            else {
                var weekToLoad = $scope.selectedWeek.week + size;
                if (weekToLoad < $scope.minWeek) return;
                if (weekToLoad > $scope.maxWeek) return;
                $scope.selectedWeek = assembleWeek(weekToLoad);
            }

        }

        function getCurrentWeek() {
            var minWeekIncomplete = 17;
            var maxWeekComplete = 0;
            if ($scope.selectedLeague && $scope.selectedLeague.matchups && $scope.selectedLeague.matchups.length > 0) {
                $scope.selectedLeague.matchups.forEach(function (matchup) {
                    if (!matchup.team_winner && matchup.team_winner !== 0) {
                        // this matchup is incomplete, compare its week to the minWeekIncomplete
                        if (matchup.week < minWeekIncomplete) {
                            minWeekIncomplete = matchup.week;
                        }
                    }
                    else {
                        if (matchup.week > maxWeekComplete) {
                            maxWeekComplete = matchup.week;
                        }
                    }
                });
            }
            var week = Math.max(minWeekIncomplete, maxWeekComplete + 1);
            return week > 17 ? 17 : week;
        }

        function assembleWeek(n) {
            var weekMatchups = {week: n, matchups: []};
            if ($scope.selectedLeague && $scope.selectedLeague.matchups && $scope.selectedLeague.matchups.length > 0) {
                $scope.selectedLeague.matchups.forEach(function(matchup) {
                    if (matchup.week === n) {
                        //poplulate the teams
                        var team1;
                        var team2;
                        $scope.selectedLeague.team.some(function(team) {
                            //only run the loop if they havent been set
                            //if ((matchup.team1 && matchup.team2) || (team1 && team2)) return true;
                            if (team1 && team2) return true;
                            if (team.team_id == matchup.team1_id) {
                                team1 = team;
                            }
                            if (team.team_id == matchup.team2_id) {
                                team2 = team;
                            }
                        });
                        matchup.team1 = team1 || { team_id: -1 };
                        matchup.team2 = team2 || { team_id: -1 };
                        if (matchup.team_winner || matchup.team_winner === 0) {
                            matchup.complete = 1;
                            matchup.team1.winner = 0;
                            matchup.team2.winner = 0;
                            if (matchup.team_winner === matchup.team1.team_id) {
                                matchup.team1.winner = 1;
                            }
                            if (matchup.team_winner === matchup.team2.team_id) {
                                matchup.team2.winner = 1;
                            }
                        }
                        weekMatchups.matchups.push(matchup);
                    }
                });
            }
            return weekMatchups;
        }

        $scope.invite = function () {
            if ($scope.selectedLeague) {
                $rootScope.pushPage('partials/leagues/invite.html', { leagueId: $scope.selectedLeague.id });
                navService.push({ title: $translate.instant('Invite Players'), controlContext: false });
            }
        };
        $scope.create = function () {
            $rootScope.pushPage('partials/leagues/create.html', { animation: 'lift' });
            navService.push({ title: $translate.instant('Create League'), controlContext: false });
        };

        $scope.revokeInvite = revokeInvite;
        function revokeInvite(invite) {
            invite.loading = 1;
            invite.$revoke(function success() {
            	removePendingInvite(invite);
            }, function error() {
            	invite.loading = 0;
            });
        }

        $scope.rejectInvite = rejectInvite;
        function rejectInvite(invite) {
            invite.loading = 1;
            invite.$reject(function success() {
                removeInvite(invite);
            }, function error() {
                invite.loading = 0;
            });
        }

        $scope.acceptInvite = acceptInvite;
        function acceptInvite(invite) {
            invite.loading = 1;
            invite.$accept(function success() {
                removeInvite(invite);
                loadLeagues(true);
            }, function error() {
                invite.loading = 0;
            });
        }

        function removePendingInvite(invite) {
        	var index = $scope.pendingInvites.map(function (u) {
        		return u.id;
        	}).indexOf(invite.id);
        	if (index > -1) {
        		$scope.pendingInvites.splice(index, 1);
        	}
        }

        function removeInvite(invite) {
            var index = $scope.invites.map(function (u) {
            	return u.inviteId;
            }).indexOf(invite.inviteId);
            if (index > -1) {
                $scope.invites.splice(index, 1);
            }
        }

        $scope.toggleRanks = toggleRanks;
        function toggleRanks() {
            $scope.ranks_closed = Math.abs($scope.ranks_closed - 1);
        }

        $scope.toggleMatchups = toggleMatchups;
        function toggleMatchups() {
        	$scope.matchups_closed = Math.abs($scope.matchups_closed - 1);
        }

        $scope.toggleInvites = toggleInvites;
        function toggleInvites() {
        	$scope.invites_closed = Math.abs($scope.invites_closed - 1);
        }

        var searchingUsers = 0;
        $scope.searchUsers = searchUsers;
        function searchUsers() {
            var value = $scope.txtUserSearch;

            //Only search after 2 chars
            if (value.length < 2) {
                return;
            }

            dataService.Users.search({ search: value.toLowerCase() },
                {},
                function success(data) {
                    //mark if already invited
                    data.forEach(function(user) {
                        if ($scope.invitees.map(function(u) {
                                return u.id;
                            })
                            .indexOf(user.id) >
                            -1) {
                            user.isInvited = true;
                        }
                    });
                    $scope.userSearchResults = data;
                },
                function error(err) {

                });
        }

        $scope.addToInvitees = addToInvitees;
        function addToInvitees(user) {
            if (!user.isInvited && $scope.invitees.map(function (u) {
                return u.id;
            }).indexOf(user.id) === -1) {
                user.isInvited = true;
                $scope.invitees.push(user);
            }
        }

        function setMinWeek(league) {
            if(league.matchups){
                $scope.minWeek = _.min(league.matchups,function(item){return item.week}).week;
            }
        }

        //---EVENTS---
        $scope.$watch('selectedLeague', function (newVal, oldVal) {
            if (!newVal) return;
            $location.search({ id: newVal.id });
            AppStateService.setLeagueId(newVal.id);
            $scope.pendingInvites = [];
            setMinWeek(newVal);
            changeWeek();
            loadExistingInvites(newVal);
        });
    }]);
