module.controller('RaceLiveResultCtrl', ['$log', '$rootScope', '$sce', '$location', '$window', '$translate', 'dataService', 'navService', function ($log, $rootScope, $sce, $location, $window, $translate, dataService, navService) {
    
    var vm = this;

    vm.error = {
		message: ""
	};
    vm.loading = {
        value: 0
    };

	var page = $rootScope.getNavigator().getCurrentPage();
	vm.currentRace = page.options.race;
	vm.hasPicks = false;
	vm.pointTotal = 0;

	function hasPicks(results) {
	    var hasPrediction = false;
	    results.forEach(function(result) {
	        if (!hasPrediction && result.pick && result.pick.predicted) hasPrediction = true;
	    });
	    return hasPrediction;
	}

    // Returns whether the given user has made a prediction for the athlete
	vm.hasPick = function (result) {
	    return result.pick && result.pick.predicted;
	}

    // Calculates the total points the user stands to gain, given current standings
	function getPointTotal(results) {
	    var pointTotal = 0;
	    results.forEach(function (result) {
	        if (result.pick && result.pick.points && parseInt(result.pick.points)) {
	            pointTotal += parseFloat(result.pick.points);
	        }
	    });
	    return pointTotal;
	}

	vm.getResults = function () {
	    vm.loading.value++;

	    dataService.Race.live({ id: vm.currentRace.race.id },
            function success(data) {
            	vm.loading.value--;
            	vm.results = data;

            	vm.hasPicks = hasPicks(vm.results);
            	if (vm.hasPicks) {
            		vm.pointTotal = getPointTotal(vm.results);
            	}

            }, function error(err) {
            	vm.loading.value--;
            	$log.debug('Error:' + JSON.stringify(err));
            	vm.error.message = err.data ? err.data.Message : 'Error retrieving live race results';
        });
	};

    vm.selectAthlete = function(athlete) {
        $rootScope.pushPage('partials/athletes/profile.html', { athleteId: athlete.id });
        navService.push({ title: $translate.instant('Profile'), controlContext: false });
    };

	vm.backToRaces = function () {
	    if ($rootScope.getNavigator().pages.length === 2) {
	        $location.search({});
	    }
	    $rootScope.popPage();
	};


	vm.viewLiveFisRace = function () {
	    //if ($rootScope.isMobile) {
	        $rootScope.openInAppBrowser(vm.currentRace.race.fis_live_results_url);
	    //}
	    //else {
	    //    $rootScope.pushPage('partials/races/live-result-fis.html', { race: vm.currentRace });
	    //    navService.push({ title: $translate.instant('Live FIS Results'), controlContext: false });
	    //}
	};

	function init() {
	    vm.getResults();
	}

	init();

}]);
