module.controller('RacePickCtrl', ['$log', '$rootScope', '$scope', '$routeParams', '$translate', 'TourService', '$location', 'AuthService', 'AccountService', 'TranslationService', 'LocalNotificationService', 'dataService', 'navService',
function ($log, $rootScope, $scope, $routeParams, $translate, TourService, $location, AuthService, AccountService, TranslationService, LocalNotificationService, dataService, navService) {
	$scope.error = {
		message: ""
	};
	$scope.pageLoading = 0;

    //get the race by id
	var page = $rootScope.getNavigator().getCurrentPage();
	$scope.userRace = page.options.race;
	$scope.userRaces = page.options.races;
    $log.debug('Race Id:' + $scope.userRace.race.id);

    //Page states enum
    var state = {PICK: 0, EDIT: 1};
    $scope.state = state.PICK;


	//Default sort property
    $scope.sortOptions = [
        { label: $translate.instant('FIS Standings'), value: 'rank' },
        { label: $translate.instant('Name'), value: 'athlete.name' },
        { label: $translate.instant('Country'), value: 'athlete.country' },
        { label: $translate.instant('Trending Picks'), value: 'trend_score' }
    ];
    $scope.sortBy = $scope.sortOptions[0];
    loadUserPicks();
    loadAthletes();


    //---IO functions----
    function loadUserPicks() {
        $scope.pageLoading++;
        dataService.Pick.get({ id: $scope.userRace.race.id },
            function success(data) {
                $scope.pageLoading--;
                $scope.userPicks = data;
                getPickCount();
                TourService.Start("racePicks");
            },
            function error(err) {
                $scope.pageLoading--;
                $log.debug('Err:' + JSON.stringify(err));
                $scope.error.message = err.data ? err.data.Message : 'Error Loading Picks';
            });
    };

    function loadAthletes() {
        $scope.pageLoading++;
        dataService.Race.queryAthletes({ id: $scope.userRace.race.id },
            function success(data) {
                $scope.allDisciplineAthletes = data;
                $scope.pageLoading--;
                //showPicksTour();

                // add bib number sort if athletes have bibs
                if ($scope.sortOptions.length == 4 && data[0].bib > 0) {
                    $scope.sortOptions.push({ label: $translate.instant('Bib Number'), value: 'bib' });
                    $scope.sortBy = $scope.sortOptions[$scope.sortOptions.length - 1];
                }
            },
            function error(err) {
                $scope.pageLoading--;
                $log.debug('Err:' + JSON.stringify(err));
                $scope.error.message = err.data ? err.data.Message : 'Error Loading Athletes';
            });
    }

    function onConfirm(index) {
        if (index === 1) { // OK button
            //push picks forward for all future races
            $rootScope.loading++;
            dataService.Pick.applyAll({ id: $scope.userRace.race.id }, $scope.userPicks, function success(data) {
                $scope.userRaces.forEach(function (race) {
                    if (!race.race.completed) {
                        if (race.race.type == $scope.userRace.race.type && race.race.stamp > $scope.userRace.race.stamp) {
                            updateRacePicks(race);
                        }
                    }
                });
                $rootScope.loading--;
            }, function error(err) {
                $rootScope.loading--;
                $rootScope.handleError();
            });
        }
    }

    //---UI functions----
    $scope.applyPicksToAll = function () {
        var title = $translate.instant('Caution!');
        var message = $translate.instant("This will apply your current picks to all future races, existing data will be overwritten.");
        var buttonLabels = [$translate.instant('Cancel'), $translate.instant('OK')];
        $rootScope.confirm(message, onConfirm, title, buttonLabels);
    };

    $scope.backToRaces = backToRaces;
    function backToRaces() {
        updateRacePicks($scope.userRace);
        if ($rootScope.getNavigator().pages.length === 2) {
            $location.search({});
        }
        $rootScope.popPage();
    }
	
	// this will reflect changes to $scope.userPicks in $scope.userRace.race
    function updateRacePicks(race) {
    	if ($scope.userPicks) {
    	    race.pickCount = getPickCount();
    	}
    	else {
    	    race.pickCount = 0;
    	}

    	LocalNotificationService.UpdateRaceNotifications([race]);
    }

    $scope.hasPicks = hasPicks;
    function hasPicks() {
    	if (!$scope.userPicks || $scope.userPicks.length == 0) return false;
    	var picks = $scope.userPicks.filter(function (element) { return element.athlete; });
    	return picks.length > 0;
    }

    $scope.toggleEditState = toggleEditState;
    function toggleEditState() {
        $scope.state = Math.abs($scope.state - 1);
    }

    $scope.isEditable = isEditable;
    function isEditable(userPick) {
        if ($scope.state != state.EDIT) return false;
        if (userPick.athlete) return true;
        return false;
    }

    //---Pick functions---
    $scope.openPickCount = 0; //zero is the falsy value to bind to
    $scope.getPickCount = getPickCount;
    function getPickCount() {
        var count = 0;
        $scope.userPicks.forEach(function (item) {
            if (item.athlete && item.athlete.id) {
                count++;
            }
        });
        $scope.openPickCount = 10 - count;
        return count;
    }

    function setLoadingAthlete(id, isLoading) {
        $scope.allDisciplineAthletes.forEach(function (item) {
            if (item.athlete.id == id) {
                item.loading = isLoading;
            }
        });
        if ($scope.trending && $scope.trending.length) {
            $scope.trending.forEach(function (position) {
                position.athletes.forEach(function (item) {
                    if (item.athlete.id == id) {
                        item.loading = isLoading;
                    }
                });
            });
        }
    }

    function setIsUserPickAthlete(id, isUserPick) {
        $scope.allDisciplineAthletes.forEach(function (item) {
            if (item.athlete.id == id) {
                item.isUserPick = isUserPick;
            }
        });
        if ($scope.trending && $scope.trending.length) {
            $scope.trending.forEach(function (position) {
                position.athletes.forEach(function (item) {
                    if (item.athlete.id == id) {
                        item.isUserPick = isUserPick;
                    }
                });
            });
        }
    }


    $scope.pickAthlete = pickAthlete;
    function pickAthlete(disciplineAthlete) {
    	if (!$scope.userRace.race.id || !$scope.openPickCount) return; //validate $scope.userRace.race.id

        // start the loading
    	setLoadingAthlete(disciplineAthlete.athlete.id, true);

    	//reduce the open pick count
        $scope.openPickCount--;

        var prepickUserPicks = JSON.parse(JSON.stringify($scope.userPicks));

    	//create the new pick
        $scope.userPicks.some(function (item, i) {
        	if (!item.athlete) {
        		item.pick_rank = i;
        		item.athlete = disciplineAthlete.athlete;
        		return true;
        	}
        })

        //save the new pick
        dataService.Pick.save({}, $scope.userPicks, function success(data) {
            //mark the athlete as picked
            setIsUserPickAthlete(disciplineAthlete.athlete.id, true);

            // stop the loading
            setLoadingAthlete(disciplineAthlete.athlete.id, false);

        }, function error(err) {

            $scope.userPicks = prepickUserPicks;

            // undo reduce the open pick count
            $scope.openPickCount++;
            // stop the loading
            disciplineAthlete.loading = false;
        });
    };

    $scope.removeAthlete = removeAthlete;
    function removeAthlete(userPick) {

        // set pick loading status
        userPick.loading = true;

        dataService.Pick.delete({},
            userPick,
            function success() {
                //mark the athlete as unpicked
                setIsUserPickAthlete(userPick.athlete.id, false);

                userPick.athlete = null;
                userPick.pick_rank = -1;
                //increase the open pick count
                $scope.openPickCount++;

                // unset pick loading status
                userPick.loading = false;
            },
            function error(err) {
                // unset pick loading status
                userPick.loading = false;
            });
    };

    $scope.reorderPick = reorderPick;
    function reorderPick(userPick, incr) {
    	// save the old picks (in case of error)
    	var prepickUserPicks = JSON.parse(JSON.stringify($scope.userPicks));

        //map to an array of ids
        var pickOrder = $scope.userPicks.map(function (item) {
            return item.athlete ? item.athlete.id : -1;
        })
        var idx = pickOrder.indexOf(userPick.athlete.id);
        //update the pick rank
        userPick.pick_rank = idx + incr + 1;

    	//reorder
        $scope.userPicks.splice(idx, 0, $scope.userPicks.splice(idx + incr, 1)[0]);

        dataService.Pick.update({}, $scope.userPicks,
            function success(data) {

            }, function error(err) {
            	//undo reorder
            	$scope.userPicks = prepickUserPicks;
            });

    }

    $scope.viewAthleteProfile = function (athlete) {
        if (athlete && athlete.id) {
            $rootScope.pushPage('partials/athletes/profile.html', { athleteId: athlete.id });
            navService.push({ title: $translate.instant('Profile'), controlContext: false });
        }
    };

    function showPicksTour() {
        // Instance the tour
        var tour = new Tour({
            //storage: window.localStorage,
            storage: false,
            autoscroll: true,
            backdrop: true,
            orphan: true,
            steps: [
                {
                    element: "#addAthlete-tour",
                    title: "Add athlete",
                    content: "Click the + to add the athlete to your top 10 predictions above.",
                    placement: 'bottom'
                },
                {
                    element: "#edit-tour",
                    title: "Edit Your Predictions",
                    content: "Use this button to toggle between editing and deleting your predictions",
                    placement: 'left'
                },
                {
                    element: "#adjustpredictions-tour",
                    title: "Adjust Predictions",
                    content: "Use the up and down arrows to adjust your predictions",
                    placement: 'bottom'
                },
                {
                    element: "#sortby-tour",
                    title: "Sort Athlete Options",
                    content: "You can sort athletes by FIS ranking, name, country, or trending picks",
                    placement: 'bottom'
                },
                {
                    element: "#pushforward-tour",
                    title: "Update all future predictions",
                    content: "If you want all your predictions to carry over for all future races of the same discipline and gender",
                    placement: 'bottom'
                }
            ],
        });

        // Initialize the tour
        tour.init();

        // Start the tour
        tour.start();
    }
}]);
