module.controller('RaceResultCtrl', ['$log', '$rootScope', '$scope', '$routeParams', '$location', '$storage', '$translate', 'TourService', 'AuthService', 'AccountService', 'TranslationService', 'AppStateService', 'dataService', 'navService', 'sportContextService',
    function ($log, $rootScope, $scope, $routeParams, $location, $storage, $translate, TourService, AuthService, AccountService, TranslationService, AppStateService, dataService, navService, sportContextService) {
    var BONUS = 0.75;
    var BONUS_CAP = 50;
    var BONUS_FINISH_MAX = 30;
    var BONUS_MIN_BIB_NUMBER = 21;
    var POINTS = [0,
        100, 80, 60, 50, 45, 40, 36, 32, 28, 26, /*1-10*/
	    24, 22, 20, 18, 16, 15, 14, 13, 12, 11, /*11-20*/
	    10, 9, 8, 7, 6, 5, 4, 3, 2, 1   /*21-30*/];
    $scope.originalStyle = false;


    $scope.error = {
		message: ""
	};
	$scope.pageLoading = 0;

	//get the race by id
	var authUserId = $storage.get('userData').id;
	var page = $rootScope.getNavigator().getCurrentPage(),
        raceId = page.options.race.race.id,
        userId = page.options.userId;
	if (!page.options.userId) {
		userId = authUserId;
	}
	$scope.userRace = page.options.race;

    loadRaceResults();

    //---IO functions----
    function loadRaceResults() {
        $scope.pageLoading++;
        dataService.UserRaceResults.get({ id: raceId, userId: userId }, function success(data) {
            $scope.result = data;

            $scope.pageLoading--;
            TourService.Start("raceResults");

            // set the selected league index
            $scope.leagueIndex = 0;
            var selectedLeagueIndex = $scope.result.LeagueRanks.map(function (u) {
                return u.Id;
            }).indexOf(AppStateService.getLeagueId());
            if (selectedLeagueIndex >= 0) {
                $scope.leagueIndex = selectedLeagueIndex;
            }

        }, function error(err) {
            $scope.pageLoading--;
            $log.debug('Err:' + JSON.stringify(err));
            $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Races';
        })
    };



	//---UI functions----
    $scope.isAuthUser = isAuthUser;
    function isAuthUser()
    {
    	return userId == authUserId;
    }

    $scope.getPickScore = function (position) {
        return $scope.getPointsEarned(position) + $scope.getPointsBonus(position);
    };

    $scope.getPointsEarned = function (position) {
        var greater = getGreater(position);

        // if in the top 30
        if (greater <= 30) {
            return POINTS[greater];
        }
        else {
            return 0;
        }
    };

    $scope.getPointsBonus = function (position) {
        var bonus = true;
        switch (sportContextService.getContext()) {
            case "JP":
            case "NK":
                // no bonus points for bib position for these sports
                return 0;
            default:
                break;
        }

        if (!position.Actual.BibNumber) {
            // No bonus
            return 0;
        }
        else {
            // (Picked athlete's Bib #) - (greater of picked position and actual finish position)
            var bonusUnits = (position.Actual.BibNumber - getGreater(position));
            if (bonusUnits > 0 && position.Actual.BibNumber >= BONUS_MIN_BIB_NUMBER && position.Actual.Position <= BONUS_FINISH_MAX) {
                // Woot, bonus!
                return Math.min(bonusUnits * BONUS, BONUS_CAP);
            }
            else {
                // No bonus
                return 0;
            }
        }
    };

    $scope.isPickGreater = function (position) {
        if (position.PickPosition == position.Actual.Position) {
            return true;
        } else {
            return getGreater(position) == position.PickPosition;
        }
    };

    function getGreater(position) {
        // greater of picked position and actual finish position
        if (position.Actual.Position > 0) {
            return position.PickPosition >= position.Actual.Position ? position.PickPosition : position.Actual.Position;
        }
        else {
            return 999;
        }
    }

    $scope.showPopup = function (id) {
        ons.createPopover('#my-popup').then(function (popover) {
            popover.show(id);
        });
    };

    $scope.showLeaders = function (leaderType) {
        var contentTitle = leaderType;
        if (leaderType === 'League') {
            contentTitle = $scope.result.LeagueRanks[$scope.leagueIndex].name;
        }
        $rootScope.pushPage('partials/races/leader-list.html', { animation: 'lift', raceId: raceId, contentTitle: contentTitle, leaderType: leaderType });
        navService.push({ title: $translate.instant('Leaderboard'), controlContext: false });
    };

    $scope.selectAthlete = function (athlete) {
        $rootScope.pushPage('partials/athletes/profile.html', { athleteId: athlete.id });
        navService.push({ title: $translate.instant('Profile'), controlContext: false });
    };

    $scope.backToRaces = function () {
        if ($rootScope.getNavigator().pages.length === 2) {
            $location.search({});
        }
        $rootScope.popPage();
    };
}]);
