module.controller('SocialCtrl', ['$log', '$rootScope', '$scope', '$routeParams', '$location', '$storage', '$timeout', '$translate', 'AuthService', 'AccountService', 'TranslationService', 'AppStateService', 'TourService', 'dataService', 'socialService', 'facebookService', 'navService',
    function ($log, $rootScope, $scope, $routeParams, $location, $storage, $timeout, $translate, AuthService, AccountService, TranslationService, AppStateService, TourService, dataService, socialService, facebookService, navService) {

        navService.initializeStack({ title: $translate.instant('Social'), controlContext: true });

        $scope.pageLoading = 0;
    	$scope.following = [];
    	$scope.followers = [];
    	$scope.search = [];
    	$scope.facebook = [];

    	$scope.error = {
            message: ""
    	};
    	var tabNames = ["", "following", "followers", "facebook", "search" ];
        $scope.tabs = {
            FOLLOWING: { tab: 1, search: {}, users: $scope.following, load: loadFollowing, placeholder: 'Filter Following', loaded: false },
            FOLLOWERS: { tab: 2, search: {}, users: $scope.followers, load: loadFollowers, placeholder: 'Filter Followers', loaded: false },
            FACEBOOK: { tab: 3, search: {}, users: $scope.facebook, load: loadFacebook, placeholder: 'Filter Friends', loaded: false },
            SEARCH: { tab: 4, search: {}, users: $scope.search, load: loadSearch, placeholder: 'Search All Users', loaded: false }
        };
        $scope.activeTab = { users: [] };

        $scope.scoreTypeOptions = [
            { id: 'Featured', text: 'Featured', icon: 'fa fa-star' },
            { id: 'Global', text: 'Global', icon: 'fa fa-globe', selected: true },
            { id: 'Country', text: 'Country', icon: 'fa fa-flag' }];
        $scope.scoreTypeSelected = 'Global';
        $scope.scoreTypeOptionsOriginalLength = $scope.scoreTypeOptions.length;

        navigateToTabContext();

        function navigateToTabContext() {
            // race context was set by a notification
            var context = AppStateService.getTabContext() || AppStateService.getSocialTab();
            if (context) {
                switch (context) {
                    case tabNames[$scope.tabs.FOLLOWERS.tab]:
                        changeTab($scope.tabs.FOLLOWERS);
                        break;
                    case tabNames[$scope.tabs.FACEBOOK.tab]:
                        changeTab($scope.tabs.FACEBOOK);
                        break;
                    case tabNames[$scope.tabs.SEARCH.tab]:
                        changeTab($scope.tabs.SEARCH);
                        break;
                    default:
                        changeTab($scope.tabs.FOLLOWING);
                        break;
                }

                // kill the tab context so this doesn't happen again
                AppStateService.setTabContext(null);
            }
            else {
                changeTab($scope.tabs.FOLLOWING);
            }
        }

        
        $scope.$watch('scoreTypeSelected', function (newValue, oldValue) {
            if (newValue != oldValue) {
                loadFollowing();
            }
        });

        // this will update the following/followers/facebook whenever a follow is added/removed
        $scope.$watch(
			function (scope) {
				return scope.activeTab.users;
			},
			function (newValue, oldValue) {
				newValue.forEach(function (result) {
					if (result.social) {
						// add/remove from following list
						var followingIndex = $scope.following.map(function (u) { return u.user_id; }).indexOf(result.user_id);
						if (followingIndex > -1) {
						    if (!result.social.following && $scope.scoreTypeSelected != 'Featured') {
								$scope.following.splice(followingIndex, 1);
							}
						}
						else {
							$scope.following.push(result);
						}

						// update the followers follow-back status
						var followerIndex = $scope.followers.map(function (u) { return u.user_id; }).indexOf(result.user_id);
						if (followerIndex > -1) {
							$scope.followers[followerIndex].social = result.social;
						}

						// update the facebookers follow-back status
						var followerIndex = $scope.facebook.map(function (u) { return u.user_id; }).indexOf(result.user_id);
						if (followerIndex > -1) {
							$scope.facebook[followerIndex].social = result.social;
						}
					}
				});
			},
			true
        );

        //---IO functions----
        function loadDisciplines() {
            if ($scope.scoreTypeOptions.length <= $scope.scoreTypeOptionsOriginalLength) {
                $scope.pageLoading++;
                dataService.Admin.disciplines(function success(disciplines) {
                    disciplines.forEach(function (discipline) {
                        $scope.scoreTypeOptions.push({ id: discipline, text: discipline });
                    });
                    $scope.pageLoading--;
                }, function error(err) {
                    $scope.pageLoading--;
                    $log.debug('Err:' + JSON.stringify(err));
                    $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Disciplines';
                });
            }
        };

        function loadFollowing() {
        	if (!$scope.following || $scope.following.length == 0 || !$scope.tabs.FOLLOWING.loaded) {
        	    $scope.pageLoading++;
        	    loadDisciplines();
        	    socialService.Follow.getFollowing({ type: $scope.scoreTypeSelected.toLowerCase() }, function success(data) {
        			$scope.following = data;
        			if (isActive($scope.tabs.FOLLOWING)) {
        				$scope.activeTab.users = $scope.following;
        			}
        			//$scope.tabs.FOLLOWING.loaded = true;
        			$scope.pageLoading--;

        			TourService.Start("social");

        		}, function error(err) {
        			$scope.pageLoading--;
        			$log.debug('Err:' + JSON.stringify(err));
        			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Following';
        		});
        	}
        }
        function loadFollowers() {
            if (!$scope.followers || $scope.followers.length == 0 || !$scope.tabs.FOLLOWERS.loaded) {
        		$scope.pageLoading++;
        		socialService.Follow.getFollowers({}, function success(data) {
        			$scope.followers = data;
        			if (isActive($scope.tabs.FOLLOWERS)) {
        				$scope.activeTab.users = $scope.followers;
        			}
        			$scope.tabs.FOLLOWERS.loaded = true;
        			$scope.pageLoading--;

        		}, function error(err) {
        			$scope.pageLoading--;
        			$log.debug('Err:' + JSON.stringify(err));
        			$scope.error.message = err.data ? err.data.Message : 'Error Retrieving Followers';
        		});
        	}
        }
        function loadFacebook() {
            checkFacebookConnected();
        }

        function loadFacebookFriends(){
            if (!$scope.facebook || $scope.facebook.length == 0 || !$scope.tabs.FACEBOOK.loaded) {
        		$scope.pageLoading++;
        		facebookService.getFriends(function (fbFriends) {
        			$scope.pageLoading--;
        			if (fbFriends.length) {
        				//now request these friends info from FSR
        				$scope.pageLoading++;
        				socialService.facebookFriends.search(
							{ friend_ids: fbFriends.map(function (item) { return item.id }) }, function success(fbFriends) {
								//mark if already followed
								fbFriends.forEach(function (result) {
									var followingIndex = $scope.following.map(function (u) { return u.user_id; }).indexOf(result.user_id);
									if (followingIndex > -1) {
										result.social = $scope.following[followingIndex].social;
									}
								});
								$scope.facebook = fbFriends;
								$scope.activeTab.users = $scope.facebook;
								$scope.tabs.FACEBOOK.loaded = true;
								$scope.pageLoading--;
							}, function error(err) {
								$scope.pageLoading--;
							})
        			}

        		});
        	}
        }

        function loadSearch() {
        	if (isActive($scope.tabs.SEARCH)) {
        		if ($scope.activeTab.search && $scope.activeTab.search.$ && $scope.activeTab.search.$.length > 1) {
        			$scope.pageLoading++;
        			socialService.Follow.search({ search: $scope.activeTab.search.$ }, function success(data) {
        				$scope.pageLoading--;
        				//mark if already followed
        				data.forEach(function (result) {
        					var followingIndex = $scope.following.map(function (u) { return u.user_id; }).indexOf(result.user_id);
        					if (followingIndex > -1) {
        						result.social = $scope.following[followingIndex].social;
        					}
        				});
        				$scope.search = data;
        				if (isActive($scope.tabs.SEARCH)) {
        					$scope.activeTab.users = $scope.search;
        				}

        			}, function error(err) {
        				$scope.pageLoading--;
        				$log.debug('Err:' + JSON.stringify(err));
        				$scope.error.message = err.data ? err.data.Message : 'Error Searching';
        			});
        			lastSearch = new Date();
        		}
        	}
        }

    	//UI functions------------
        $scope.facebookConnected = true;
        function checkFacebookConnected() {
        	$scope.pageLoading++;
        	AuthService.getFacebookConnected(function success(connectedStatus) {
        		$scope.pageLoading--;
        		$scope.facebookConnected = connectedStatus;
        		if (connectedStatus) {
        			loadFacebookFriends();
        		}
        		//$scope.safeApply();
        	}, function error() {
        		$scope.pageLoading--;
            	$scope.facebookConnected = false;
            });
        }


        var THROTTLE = 500;
        var lastSearch;
        var nextSearchFn;
        $scope.doSearch = doSearch;
        function doSearch() {
        	if (lastSearch && (new Date() - lastSearch < THROTTLE)) {
        		//if there is already a search, cancel it
        		if (nextSearchFn) clearTimeout(nextSearchFn);
        		nextSearchFn = setTimeout(function () {
        			loadSearch();
        		}, THROTTLE);
        	}
        	else {
        		loadSearch();
        	}
        }

        $scope.changeTab = changeTab;
        function changeTab(tab) {
            if (!$scope.activeTab || tab.tab != $scope.activeTab.tab) {
        	    // set the new tab
        	    $scope.activeTab = tab;
        		tab.load();
        	}
        }
        $scope.isActive = isActive;
        function isActive(tab) {
        	return tab.tab == $scope.activeTab.tab;
        }


        $scope.$watch('activeTab', function (newVal, oldVal) {
            if (!newVal) return;
            $timeout(function () {
                var name = tabNames[newVal.tab];
                AppStateService.setSocialTab(name);
                var currentName = $location.search().id;
                $location.search({ id: name });
            }, 10);
        });

        //Events
        $scope.$on('facebook-connected', function () {
        	checkFacebookConnected();
        });
    }]);
