module.controller('UserRacesCtrl', ['$log', '$rootScope', '$scope', '$routeParams', '$location', '$translate', 'AuthService', 'AccountService', 'TranslationService', 'AppStateService', 'TourService', 'LocalNotificationService', 'dataService', 'navService',
	function ($log, $rootScope, $scope, $routeParams, $location, $translate, AuthService, AccountService, TranslationService, AppStateService, TourService, LocalNotificationService, dataService, navService) {
	$scope.error = {
		message: ""
	};
	$scope.pageLoading = 0;

	navService.initializeStack({ title: $translate.instant('Races'), controlContext: true });
        
	$scope.$on("notification:local:race", function (event, notification) {
	    if ($scope.userRaces && $scope.userRaces.length > 0) {
	        navigateToTabContext();
	    }
	});

	function navigateToTabContext() {
	    // race context was set by a notification
	    var context = AppStateService.getTabContext();
	    if (context) {
	        var userRace = $scope.userRaces.filter(function (race) {
	            return race.race.id === context;
	        });
	        if (userRace.length > 0) {
	            $scope.selectRace(userRace[0]);
	        }

	        // kill the tab context so this doesn't happen again
	        AppStateService.setTabContext(null);
	    }
	}

    //---IO functions----
    loadRaces();
    function loadRaces(){
        $scope.pageLoading++;
        dataService.UserRace.query(function success(races){
            $scope.userRaces = races;

            LocalNotificationService.UpdateRaceNotifications($scope.userRaces);

            scrollToRecentRace();

            navigateToTabContext();

            $scope.pageLoading--;

            TourService.Start("races");

        },function error(err){
            $scope.pageLoading--;
            $log.debug('Err:' + JSON.stringify(err));
            $scope.error.message = err.data ? err.data.Message : 'Error Retrieving Races';
        });
    };

    //UI functions------------
    $scope.closestRacePosition = null;
    function scrollToRecentRace() {
        //determine the height of the elements
        var distance;
        var closestRace = null;
        var currentDate = new Date();
        $scope.userRaces.forEach(function (item) {
            var thisDistance = Math.abs(new Date(Date.parse(item.race.race_date)) - currentDate);
            if (!distance || thisDistance < distance) {
                distance = thisDistance;
                closestRace = item;
            }
        })
        var position = $scope.userRaces.indexOf(closestRace) + 1;
        // get item height, no jQuery
        $scope.closestRacePosition = position;
        //showRacesTour();
    };
	
    $scope.selectRace = function (userRace) {
        $location.search({ id: userRace.race.id });
        // 1 - Completed race - Review
        if (userRace.race.completed) {
            $rootScope.pushPage('partials/races/result.html', { race: userRace });
            navService.push({ title: $translate.instant('Results'), controlContext: false });
        }
        // 2 - Race in progress - Follow
        else if (userRace.isLive && !userRace.race.completed) {
            $rootScope.pushPage('partials/races/live-result.html', { race: userRace });
            navService.push({ title: $translate.instant('Live Results'), controlContext: false });
        }
        // 3 - Race has yet to occur - Pick
        else {
            $rootScope.pushPage('partials/races/pick.html', { race: userRace, races: $scope.userRaces });
            navService.push({ title: $translate.instant('My Picks'), controlContext: false });
        }
    };
}]);
