module.directive('fsrAthleteChart', ['$translate', '$filter', function ($translate, $filter) {
    return {
        restrict: 'E',
        replace: false,
        template: '<div ng-show="resultsData"><canvas id="history-chart" class="chart chart-bar" chart-data="results.data" chart-labels="results.labels" chart-series="results.series" chart-options="results.options" chart-dataset-override="results.datasetOverride"></canvas></div>',
        transclude: false,
        scope: {
            resultsData: "="
        },
        link: function (scope, element, attrs) {
            scope.$watch(
		        function (scope) {
		            return scope.resultsData;
		        },
		        function (newValue, oldValue) {
		            if (newValue && newValue !== oldValue) {
		                scope.update(newValue.races);
		            }
		        });
            scope.update = buildData;

            function buildData (data) {
                scope.results = {};
                scope.results.series = ['Recent Results'];
                scope.results.options = getChartOptions();
                setChartData(data);
                scope.results.datasetOverride = [
                    {
                        backgroundColor: "rgba(30, 117, 187, 0.2)", // Fill color under line
                        borderColor: "rgba(30, 117, 187, 1)", // Color of the line
                        hoverBackgroundColor: "rgba(30, 117, 187, 1)"
                    }
                ];

            };

            function getMaxPlacement(data) {
                return parseInt(data.maxPlacement) + 4;
            }

            function setChartData(data) {
                
                //convert positions from strings to numbers (when not DNF)
                data = data.map(function (item){
                    item.position = parseInt(item.position) ? parseInt(item.position) : item.position;
                    return item;
                });

                scope.results.labels = data.map(function (item) {
                    return $filter('date')(item.date, "mediumDate");
                });

                // Offset value for max (worst) possible placement for the athlete within some set of races
                // Necessary to get around how small the bar is for low placements
                var maxPlacementOffset = getMaxPlacement(scope.resultsData);

                // replace non-numeric position values with dnf (n+1, where n is the highest/worst finish in the set)
                scope.results.data = [data.map(function (item) {
                    if (parseInt(item.position)) {
                        return maxPlacementOffset - item.position;
                    } else {
                        // DNF or other strings
                        return 3;
                    }
                })];

            };

            function getChartOptions() {
                return {
                    responsive: true,
                    tooltips: {
                        enabled: true,
                        backgroundColor: "rgba(30,117,187,0.9)",
                        xPadding: 5,
                        yPadding: 5,
                        titleFontSize: 13,
                        bodyFontSize: 15,
                        bodyFontStyle: "normal",
                        bodyFontColor: "#fff",
                        bodySpacing: 13,
                        footerFontSize: 13,
                        callbacks: {
                            label: function (tooltipItems, data) {
                                var race = scope.resultsData.races[tooltipItems.index];
                                return race.position;
                            },
                            footer: function (tooltipItems, data) {
                                var race = scope.resultsData.races[tooltipItems[0].index];
                                return race.eventName + ' - ' + race.discipline_id;
                            }

                        }
                    },
                    scales: {
                        xAxes: [{
                            categoryPercentage: 0.9,
                            gridLines: {
                                display: false
                            },
                            ticks: {
                                display: false
                            },
                            display: false
                        }],
                        yAxes: [{
                            gridLines: {
                                display: false
                            },
                            ticks: {
                                display: false,
                                min: 0,
                                max: getMaxPlacement(scope.resultsData)
                                //reverse: true
                            },
                            display: false
                        }]
                    },
                    ticks: {
                        display: false
                    }

                };
            };

            function init() {
                buildData(scope.resultsData.races);
            }

            init();

        }
    };
}]);