module.directive('fsrChallengeItem', ['$timeout', function ($timeout) {
    return {
        restrict: 'E',
        replace: false,
        templateUrl: './templates/directives/fsr-challengeItem.html',
        scope: {
            challenge:'=', joined:'=', details: '='
        },
        controller: ['$rootScope', '$scope', '$location', 'AppStateService', 'navService', function ($rootScope, $scope, $location, AppStateService, navService) {

            function navigateToTabContext() {
                // challenge context was set by a notification
                var context = AppStateService.getTabContext();
                if (context) {
                    if ($scope.challenge.id === context) {
                        // kill the tab context so this doesn't happen again
                        AppStateService.setTabContext(null);

                        $scope.view($scope.challenge);
                    }
                }
            }

            $scope.$on("notification:local:challenge", function (event, notification) {
                navigateToTabContext();
            });

            $scope.view = function (challenge) {
                var currentPage = $rootScope.getNavigator().getCurrentPage();
                if (currentPage.options && currentPage.options.challenge && currentPage.options.challenge.id == challenge.id) {
                    // don't do anything, the current challenge is already at the top of the display stack
                }
                else {
                    $location.search({ id: challenge.id });
                	if (challenge.club_id) {
                	    $rootScope.pushPage('partials/challenges/details-club.html', { challenge: challenge });
                	    navService.push({ title: challenge.name, controlContext: false });
                	} else {
                	    $rootScope.pushPage('partials/challenges/details.html', { challenge: challenge });
                	    navService.push({ title: challenge.name, controlContext: false });
                	}
                }
        	};

        	$scope.isJoined = function () {
        	    return $scope.challenge.joined;
        	}

        	$scope.challenge.goalPct = $scope.challenge.goal && $scope.challenge.score < $scope.challenge.goal ? $scope.challenge.score * 100 / $scope.challenge.goal : 100;
        	$scope.challenge.joined = $scope.challenge.joined || $scope.joined;

            // do this when the challenge is loaded
        	navigateToTabContext();
        }],
        link: function (scope, element, attrs) {
        }
    };
}]);