module.directive('fsrUserChart', ['$translate', '$filter', function ($translate, $filter) {
    return {
        restrict: 'E',
        replace: false,
        template: '<div ng-if="races"><canvas id="race-chart" class="chart chart-bar" chart-data="races.data" chart-labels="races.labels" chart-series="races.series" chart-options="races.options" chart-dataset-override="races.datasetOverride"></canvas></div>',
        transclude: false,
        scope: {
            raceData: "="
        },
        link: function (scope, element, attrs) {
            // replace the necessary tags with the attributes passed in
            
           

            function getItemsToShow()
            {
                //element parent
                var width = element[0].parentElement.clientWidth;
                var itemsToShow = width / 30;
                return itemsToShow;
            };

            function buildRaceData (raceData, options) {
                scope.races = {};
                scope.races.series = ['Recent Results'];
                scope.races.options = options || getRaceChartOptions();
                setRaceChartData(raceData, getItemsToShow());
                scope.races.datasetOverride = [
                    {
                        label: "Race Results",
                        backgroundColor: "rgba(30, 117, 187, 0.2)", // Fill color under line
                        borderColor: "rgba(30, 117, 187, 1)", // Color of the line
                        hoverBackgroundColor: "rgba(30, 117, 187, 1)"
                    }
                    /*
                    {
                        label: "Race Results",
                        hoverBackgroundColor: "rgba(255,99,132,0.4)",
                        backgroundColor: ["rgba(30, 117, 187, 0.2)"], // Fill color under line
                        fill: "rgba(30, 117, 187, 0.2)", // If true, fill area under line
                        borderColor: "rgba(30, 117, 187, 1)", // Color of the line
                        pointBackgroundColor: "rgba(30, 117, 187, 1)", // The fill color for points
                        pointBorderColor: "#ffffff", // The border color for points
                        pointHoverBackgroundColor: "#ffffff", // Point background color when hovered
                        pointHoverBorderColor: "rgba(30, 117, 187, 1)" //  Point border color when hovered

                    }
                    */
                ];

            };

            function setRaceChartData(data, itemsToShow) {
                itemsToShow = itemsToShow || 20;

                var allCompleted = data.filter(function (item) { return item.race.completed; });
                var startIndex = Math.max(0, allCompleted.length - itemsToShow);
                scope.completedRaceData = allCompleted.slice(startIndex, allCompleted.length);
                /*
                var labels = completed.map(function (item) {
                    var race = item.race.location + ' ' + item.race.type.substring(0, 2);
                    // this is the same as the dataPoints
                    var score = item.race.picks && item.race.picks.length ? item.race.picks[0].points || 0 : 0;
                    return race + ': ' + $filter('number')(score, 2);
                });
                var dataPoints = completed.map(function (item) { return item.race.picks && item.race.picks.length ? item.race.picks[0].points || 0 : 0; });
                */
                var dataSet = scope.completedRaceData.map(function (item) {
                    var race = (item.race.location !== null ? item.race.location : '')  + ' ' + item.race.discipline_id;
                    var score = item.pick ? item.pick.points || 0 : 0;
                    return { 'x': race, 'y': score };
                });

                scope.races.labels = dataSet.map(function (item) { return item.x; });
                scope.races.data = [dataSet.map(function (item) { return item.y; })];
                
            };

            function getRaceChartOptions() {
                return {
                    responsive: true,
                    tooltips: {
                        enabled: true,
                        backgroundColor: "rgba(30,117,187,0.9)",
                        xPadding: 10,
                        yPadding: 10,
                        titleFontSize: 16,
                        bodyFontSize: 24,
                        bodyFontStyle: "normal",
                        bodyFontColor: "#fff",
                        bodySpacing: 16,
                        callbacks: {
                            label: function (tooltipItems, data) {
                                var race = scope.completedRaceData[tooltipItems.index];
                                var score = race.pick  ? race.pick.points || 0 : 0;
                                return $filter('number')(score, 2);
                            }
                        }
                    },
                    elements: {
                        line: {
                            borderColor: "rgba(30, 117, 187, 1)",
                            borderWidth: 2
                        },
                        point: {
                            radius: 5,
                            hitRadius: 10,
                            backgroundColor: "rgba(30,117,187,1)",
                            borderColor: "rgba(255,255,255,1)",
                        }
                    },
                    scales: {
                        xAxes: [{
                            gridLines: {
                                display: false
                            },
                            ticks: {
                                display: false
                            },
                            display: false
                        }],
                        yAxes: [{
                            gridLines: {
                                display: false
                            },
                            ticks: {
                                display: false
                            },
                            display: false
                        }]
                    },
                    ticks: {
                        display: false
                    }

                };
            };

            function init() {
                buildRaceData(scope.raceData);
            }

            init();

        }
    };
}]);