﻿(function () {
    'use strict';

    /**
    *
    *Root module
    */
    angular.module('ga.tracking', []);

    /**
    *
    *Service to set u meta data to attach to analytics post data
    */
    angular.module('ga.tracking').service('metaSVC', metaservice);

    function metaservice() {
        var _client_id = null;

        /**
        *
        *Generate GUID 
        */
        this.getClientId = function () {
            try {
                if (!!_client_id) {                                                                      //check for null or undefined
                    return _client_id;
                }
                _client_id = generateClientId();
            }
            catch (err) { console.log(err); }
            return _client_id;
        }

        /**
        *
        *Generate GUID 
        */
        function generateClientId() {
            _client_id = generateGUID();
            //_client_id = generateDate() + "-" + generateGUID();                                     //Piece together our unique id for this device
            return _client_id;
        }

        /**
        *
        *Generate GUID 
        */
        function generateGUID() {
            var GUID = (gen4Set() + gen4Set() + "-" + gen4Set() + "-4" + gen4Set().substr(0, 3) + "-" + gen4Set() + "-" + gen4Set() + gen4Set() + gen4Set()).toLowerCase();
            return GUID;
        }

        /**
        *
        *Generate GUID 
        */
        function gen4Set() {
            return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);                       //0x10000 hex version of 65536
        }

        /**
        *
        *Generate GUID 
        */
        function generateDate() {
            var currentTime = Date.now() || +new Date();                                            //use Date.now()  so we don't create an unecessary date object.
            return currentTime;
        }

        /**
        *
        *Generate a random integer 
        */
        this.getRandomInt = function (min, max) {
            return Math.floor(Math.random() * (max - min + 1)) + min;
        }
    }

    /*
    Use :
    pageview :  gaTrackingService.pageTrack("name_of_view");
    event  : gaTrackingService.eventTrack("name_of_view","name_of_event_action","name_of_event_category");
    Read more here : Google analytics: https://developers.google.com/analytics/devguides/collection/protocol/v1/parameters
    */

    angular.module('ga.tracking')
    .service('gaTrackingService', ['$http', 'metaSVC', gaTrackingService]);

    function gaTrackingService($http, metaSVC) {

        /**********************************************************************************************
        *
        * Be sure to chagne the data below to appropriate data from Google Analytics Dashboard
        * appglobals could be provided as a factory as well, supply these parameters however you wish
        * 
        ********************************************************************************************* */
        var appglobals = {
            ga_root: "http://www.google-analytics.com/collect?v=1",
            ga_root_ssl: "https://ssl.google-analytics.com/collect?v=1",
            ua: "UA-27378356-1",
            appname: "FSR Top 10",
            av: "1.4.5"
        };


        this.pageTrack = function (view) {
            sendupcall(view, false, "", "");
        };

        this.eventTrack = function (event, eventaction, eventcategory) {
            sendupcall(event, true, eventaction, eventcategory);
        };

        /**
        *
        *@param view (page) you wish to send up in post data
        *@param flag t determine whether this regular page view or user event
        *@param event action to attach to post
        *@param event category to attach to post
        */
        function sendupcall(view, isevent, eventaction, eventcategory) {
            try {
                var _url = buildUrl(view, isevent, eventaction, eventcategory);
                $http.post(_url)
                .success(function (data, status) {
                    //console.log("GA call completed ...");
                })
                .error(function (data) {
                    //console.log("GA call failed ...");
                });
            }
            catch (err) { console.log(err); }
        }

        /*
        * isevent set to true to send up google event , set to false  to send up google pageview
        * Read more here : Google analytics: https://developers.google.com/analytics/devguides/collection/protocol/v1/parameters
        */
        function buildUrl(view, isevent, eventaction, eventcategory) {
            var url = appglobals.ga_root_ssl;
            url = url + "&cid=" + metaSVC.getClientId();
            url = url + "&tid=" + appglobals.ua;
            //url = url + "&dl=" + encodeURIComponent(document.location.origin + document.location.pathname + document.location.search);
            //url = url + "&dh=" + encodeURIComponent('fantasyskiracer.com');
            url = url + "&dp=" + encodeURIComponent(view);
            url = url + "&dt=" + encodeURIComponent(view);
            url = url + "&cd=" + encodeURIComponent(view);
            url = url + "&an=" + encodeURIComponent(appglobals.appname);
            url = url + "&av=" + encodeURIComponent(appglobals.av);

            if (isevent) {
                url = url + "&ec=" + encodeURIComponent(eventcategory);
                url = url + "&ea=" + encodeURIComponent(eventaction);
                url = url + "&t=event";
            }
            else {
                url = url + "&t=pageview";
            }
            if (typeof cordova != 'undefined') {
                url = url + "&ds=app";
            }
            else {
               url = url + "&ds=web";
            }
            url = url + "&z=" + metaSVC.getRandomInt(1, 1000000);
            return url;
        }
    }

})();