services.factory('dataService', ['$rootScope', '$http', '$resource', '$q', '$storage', 'appSettings', function ($rootScope, $http, $resource, $q, $storage, appSettings) {

    //----Private----
    var
        _account = $resource(appSettings.serviceBase + '/api/account', null, {
            'get': { method: 'GET' },
            'save': { method: 'POST', url: appSettings.serviceBase + '/api/account/register' },
            'update': { method: 'PUT' },
            'updatepassword': { method: 'PUT', url: appSettings.serviceBase + '/api/account/updatepassword' }
        }),
		 _ad = $resource(appSettings.serviceBase + '/api/ads/:id', { id: '@id' }, {
		     'get': { method: 'GET', url: appSettings.serviceBase + '/api/ads/all', isArray: true },
		     'save': { method: 'POST', url: appSettings.serviceBase + '/api/ads' },
		     'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/ads/:id', params: { id: '@id' } }
		 }),
		_admin = $resource(appSettings.serviceBase + '/api/admin/', null, {
		    'disciplines': { method: 'GET', url: appSettings.serviceBase + '/api/admin/disciplines', isArray: true },
		    'equipments': { method: 'GET', url: appSettings.serviceBase + '/api/admin/equipment', isArray: true },
		    'socials': { method: 'GET', url: appSettings.serviceBase + '/api/admin/social', isArray: true },
		    'uploadRaceResults': { method: 'POST', url: appSettings.serviceBase + '/api/admin/races/results', params: { model: '@json' } },
		    'processRaceResults': { method: 'GET', url: appSettings.serviceBase + '/api/admin/races/results/:id', params: { id: '@id' } },
		    'updateRankings': { method: 'GET', url: appSettings.serviceBase + '/api/admin/athletes/rank' },
		}),
    	 _athlete = $resource(appSettings.serviceBase + '/api/athletes/:id', { id: '@id' }, {
    	     'save': { method: 'POST', url: appSettings.serviceBase + '/api/athletes' },
    	     'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/athletes/:id', params: { id: '@id' } },
    	     'query': { method: 'GET', url: appSettings.serviceBase + '/api/athletes', isArray: true },
    	     'history': { method: 'POST', url: appSettings.serviceBase + '/api/athletes/history' },
    	     'getLocations': { method: 'GET', url: appSettings.serviceBase + '/api/athletes/:id/locations', params: { id: '@id' }, isArray: true }
    	 }),
    	 _challenge = $resource(appSettings.serviceBase + '/api/challenges/:id', { id: '@id' }, {
    	     'get': { method: 'GET', url: appSettings.serviceBase + '/api/challenges', isArray: true },
    	     'getRaces': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/:id/races', params: { id: '@id' }, isArray: true },
    	     'overall': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/:id/leaders/overall', params: { id: '@id' }, isArray: true },
    	     'following': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/:id/leaders/following', params: { id: '@id' }, isArray: true },
    	     'myClub': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/:id/leaders/club/:clubid', params: { id: '@id', clubid: '@clubid' } },
    	     'competingClubs': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/:id/leaders/clubs', params: { id: '@id' }, isArray: true },
    	     'save': { method: 'POST', url: appSettings.serviceBase + '/api/challenges' },
    	     'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/challenges/:id', params: { id: '@id' } },
    	     'query': { method: 'GET', url: appSettings.serviceBase + '/api/challenges/user/:curdate', params: { curDate: '@curdate' }, isArray: true },
    	     'join': { method: 'PUT', url: appSettings.serviceBase + '/api/challenges/:id/join', params: { id: '@id' } },
    	     'leave': { method: 'PUT', url: appSettings.serviceBase + '/api/challenges/:id/leave', params: { id: '@id' } },
    	 }),
    	 _club = $resource(appSettings.serviceBase + '/api/clubs/:id', { id: '@id' }, {
    	     'save': { method: 'POST', url: appSettings.serviceBase + '/api/clubs' },
    	     'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/clubs/:id', params: { id: '@id' } },
    	     'leaders': { method: 'GET', url: appSettings.serviceBase + '/api/clubs/:id/leaders', params: { id: '@id' } },
    	     'query': { method: 'GET', url: appSettings.serviceBase + '/api/clubs', isArray: true },
    	     'queryMembers': { method: 'GET', url: appSettings.serviceBase + '/api/clubs/:id/members', params: { id: '@id' }, isArray: true },
    	     'getByUssaId': { method: 'GET', url: appSettings.serviceBase + '/api/clubs/ussa/:id', params: { id: '@id' }, isArray: true },
    	 }),
        _invites = $resource(appSettings.serviceBase + '/api/invites/:id', { id: '@id' }, {
            //CAPTAIN - OWNER
            'getPendingInvites': { method: 'GET', url: appSettings.serviceBase + '/api/leagues/:leagueId/invites', isArray: true },
            'sendInvites': { method: 'POST', url: appSettings.serviceBase + '/api/leagues/:leagueId/invites', params: { leagueId: '@leagueId' }, isArray: false },
            'revoke': { method: 'POST', url: appSettings.serviceBase + '/api/leagues/:leagueId/invites/:id/revoke', params: { leagueId: '@leagueId', id: '@id' }, isArray: false },
            //USER
            'get': { method: 'GET', url: appSettings.serviceBase + '/api/users/:id/invites', params: { id: '@id' }, isArray: true },
            'accept': { method: 'POST', url: appSettings.serviceBase + '/api/leagues/:leagueId/invites/:inviteId/accept', params: { leagueId: '@leagueId', inviteId: '@inviteId' }, isArray: false },
            'reject': { method: 'POST', url: appSettings.serviceBase + '/api/leagues/:leagueId/invites/:inviteId/reject', params: { leagueId: '@leagueId', inviteId: '@inviteId' }, isArray: false }
        }),
        _leagues = $resource(appSettings.serviceBase + '/api/leagues/:id', { id: '@id' }, {
            'removeTeam': { method: 'DELETE', url: appSettings.serviceBase + '/api/leagues/:leagueId/teams/:teamId', params: { leagueId: '@leagueId', teamId: '@teamId' }, isArray: false }
        }),
        _pick = $resource(appSettings.serviceBase + '/api/picks/:id', { id: '@id' }, {
            'get': { method: 'GET', isArray: true },
            'save': { method: 'POST', isArray: true },
            'applyAll': { method: 'POST', url: appSettings.serviceBase + '/api/picks/:id/future' },
            'update': { method: 'POST', isArray: true },
            'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/picks/:raceId/:athleteId', params: { raceId: '@raceId', athleteId: '@athlete.id' } }
        }),
        _race = $resource(appSettings.serviceBase + '/api/races/:id', { id: '@id' }, {
            'save': { method: 'POST', url: appSettings.serviceBase + '/api/races' },
            'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/races/:id', params: { id: '@id' } },
            'query': { method: 'GET', url: appSettings.serviceBase + '/api/races', isArray: true },
            'queryAthletes': { method: 'GET', url: appSettings.serviceBase + '/api/races/:id/athletes', params: { id: '@_id' }, isArray: true },
            'queryTrendingAthletes': { method: 'GET', url: appSettings.serviceBase + '/api/races/:id/athletes/trending', params: { id: '@_id' }, isArray: true },
            'live': { method: 'GET', url: appSettings.serviceBase + '/api/races/:id/live', params: { id: '@id' }, isArray: true }
        }),
        _sport = $resource(appSettings.serviceBase + '/api/sports', { }, {
        	'get': { method: 'GET', isArray: true }
        }),
		_stats = $resource(appSettings.serviceBase + '/api/stats/:id', { id: '@id' }, {
		    'memberCount': { method: 'GET', url: appSettings.serviceBase + '/api/statsMembers' },
		    'leagueCount': { method: 'GET', url: appSettings.serviceBase + '/api/statsLeauges' },
		    'clubsCount': { method: 'GET', url: appSettings.serviceBase + '/api/statsClubs' }
		}),
        _teams = $resource(appSettings.serviceBase + '/api/leagues/:id', { id: '@id' }, {
            'remove': { method: 'DELETE', url: appSettings.serviceBase + '/api/leagues/:leagueId/teams/:teamId', params: { leagueId: '@leagueId', teamId: '@teamId' }, isArray: false }
        }),
		_tour = $resource(appSettings.serviceBase + '/api/tours/:id', { id: '@id' }, {
		    'query': { method: 'GET', url: appSettings.serviceBase + '/api/tours/all', isArray: true },
		    'get': { method: 'GET', url: appSettings.serviceBase + '/api/tours/:id', params: { id: '@id' } },
		    'save': { method: 'POST', url: appSettings.serviceBase + '/api/tours' },
		    'delete': { method: 'DELETE', url: appSettings.serviceBase + '/api/tours/:id', params: { id: '@id' } }
		}),
        _userRace = $resource(appSettings.serviceBase + '/api/userraces/:id'),
        _userRaceResults = $resource(appSettings.serviceBase + '/api/results/:id/:userId', { id: '@id', userId: '@userId' }, {
            'leaders': { method: 'GET', url: appSettings.serviceBase + '/api/results/:id/leaders/:leaderType', params: { id: '@id', leaderType: '@leaderType', leagueId: '@leagueId' }, isArray: true },
            'compare': { method: 'GET', url: appSettings.serviceBase + '/api/results/week/:week/details', params: { week: '@week', user1: '@user1', user2: '@user2' } }
        }),
        _users = $resource(appSettings.serviceBase + '/api/users/:id', { id: '@id' }, {
            'search': { method: 'GET', url: appSettings.serviceBase + '/api/users/search', isArray: true },
            'getLatest': { method: 'GET', url: appSettings.serviceBase + '/api/users/getlatest/:sinceDate', params: { sinceDate: '@sinceDate' }, isArray: true },
            'getTotals': { method: 'GET', url: appSettings.serviceBase + '/api/users/gettotals/:sinceDate', params: { sinceDate: '@sinceDate' } },
            'getPickLatest': { method: 'GET', url: appSettings.serviceBase + '/api/users/getpicklatest/:raceId/:sinceDate', params: { raceId: '@raceId', sinceDate: '@sinceDate' }, isArray: true },
            'getPickTotals': { method: 'GET', url: appSettings.serviceBase + '/api/users/getpicktotals/:raceId', params: { raceId: '@raceId' } }
        });

    //----------------

    //----Public----
    return {
        Account: _account,
        Ad: _ad,
        Admin: _admin,
        Athlete: _athlete,
        Challenge: _challenge,
        Club: _club,
        Invites: _invites,
        Leagues: _leagues,
        Pick: _pick,
        Race: _race,
		Sport: _sport,
        Stats: _stats,
        Teams: _teams,
        Tour: _tour,
        UserRace: _userRace,
        UserRaceResults: _userRaceResults,
        Users: _users
    };
    //-------------
}]);
