﻿'use strict';
var services = angular.module('fsrTopTenServices', ['ngResource', 'angularLocalStorage']);
services.factory('AppStateService', ['$rootScope', '$http', '$q', '$storage', '$translate', 'tmhDynamicLocale', function ($rootScope, $http, $q, $storage, $translate, tmhDynamicLocale) {
    var appStateServiceFactory = {};

    function getState() {
        var state = $storage.get('appState') || {};
        var extendedState = angular.extend({}, {
            tabId: 1, // races tab
            tabContext: null, // none
            leagueId: null, // none
            socialTab: null,
            depth: 0 // none
        }, state);
        return extendedState;
    }

    function saveState(state) {
        $storage.set('appState', state);
    }

    var _setTab = function (tabId) {
        if (tabId >= 0) {
            var state = getState();
            state.tabId = tabId;
            saveState(state);
        }
    };

    var _getTab = function () {
        return getState().tabId;
    };

    var _setTabContext = function (context) {
        var state = getState();
        state.tabContext = context;
        saveState(state);
    };

    var _getTabContext = function () {
        return getState().tabContext;
    };

    var _setLeagueId = function (leagueId) {
        if (leagueId) {
            var state = getState();
            state.leagueId = leagueId;
            saveState(state);
        }
    };

    var _getLeagueId = function () {
        return getState().leagueId;
    };

    var _setSocialTab = function (tab) {
        if (tab) {
            var state = getState();
            state.socialTab = tab;
            saveState(state);
        }
    };

    var _getSocialTab = function () {
        return getState().socialTab;
    };

    var _setDepth = function (depth) {
        var state = getState();
        state.depth = depth;
        saveState(state);
    };

    var _getDepth = function () {
        return getState().depth || 0;
    };

    appStateServiceFactory.setTab = _setTab;
    appStateServiceFactory.getTab = _getTab;
    appStateServiceFactory.setTabContext = _setTabContext;
    appStateServiceFactory.getTabContext = _getTabContext;
    appStateServiceFactory.setLeagueId = _setLeagueId;
    appStateServiceFactory.getLeagueId = _getLeagueId;
    appStateServiceFactory.setSocialTab = _setSocialTab;
    appStateServiceFactory.getSocialTab = _getSocialTab;
    appStateServiceFactory.setDepth = _setDepth;
    appStateServiceFactory.getDepth = _getDepth;
    return appStateServiceFactory;
}]);

services.factory('TranslationService', ['$rootScope', '$http', '$q', '$storage', '$translate', 'tmhDynamicLocale', function ($rootScope, $http, $q, $storage, $translate, tmhDynamicLocale) {
	var translationServiceFactory = {};
	$storage.bind($rootScope, 'lang', $translate.use());

	var _changeLanguage = function (lang) {
		if (lang) {
			$storage.set('lang', lang);
			$translate.use(lang);
			if (lang.length >= 2) {
				tmhDynamicLocale.set(lang.substring(0, 2));
			}
		}
	};

	var _getLanguage = function () {
		return $storage.get('lang') || $translate.use();
	};

	translationServiceFactory.changeLanguage = _changeLanguage;
	translationServiceFactory.getLanguage = _getLanguage;
	translationServiceFactory.getLanguages = function () {
		return $http.get('js/libs/l10n/languages.json');
	};
	translationServiceFactory.getCountries = function () {
		return $http.get('js/libs/l10n/countries.json');
	};

	_changeLanguage(_getLanguage());

	return translationServiceFactory;
}]);

services.factory('AnalyticsService', ['$analytics', '$storage', function ($analytics, $storage) {
	var analyticsServiceFactory = {};

	var _setUsername = function (userinfo) {
		if (userinfo && userinfo.username) {
			var properties = {
				"$email": userinfo.email,
				"$created": userinfo.created,
				"$last_login": new Date(),
				"Name": userinfo.name,
				"Gender": userinfo.gender,
				"DOB": userinfo.dob
			};

			$analytics.setUsername(userinfo.username);
			$analytics.setUserProperties(properties);
		}
	};

	var _eventTrack = function (event, properties) {
		$analytics.eventTrack(event, properties);
	};

	analyticsServiceFactory.setUsername = _setUsername;
	analyticsServiceFactory.eventTrack = _eventTrack;

	return analyticsServiceFactory;
}]);

services.factory('UploadService', ['$rootScope', '$http', '$q', 'appSettings', function ($rootScope, $http, $q, appSettings) {
	var uploadServiceFactory = {};

	var _saveChallengeImage = function (challengeId, file) {
		var deferred = $q.defer();
		$http({
			method: 'POST',
			url: appSettings.serviceBase + '/api/challenges/image/' + challengeId,
			//IMPORTANT!!! You might think this should be set to 'multipart/form-data' 
			// but this is not true because when we are sending up files the request 
			// needs to include a 'boundary' parameter which identifies the boundary 
			// name between parts in this multi-part request and setting the Content-type 
			// manually will not set this boundary parameter. For whatever reason, 
			// setting the Content-type to 'undefined' will force the request to automatically
		    // populate the headers properly including the boundary parameter.
			headers: { 'Content-Type': undefined },
			//This method will allow us to change how the data is sent up to the server
			// for which we'll need to encapsulate the model data in 'FormData'
			transformRequest: function (data) {
				var formData = new FormData();
				formData.append("file", data.file);
				
				return formData;
			},
			//Create an object that contains the model and files which will be transformed
			// in the above transformRequest method
			data: { file: file }
		}).
		success(function (data, status, headers, config) {
			deferred.resolve(data);
		}).
		error(function (data, status, headers, config) {
			deferred.reject("Could not upload challenge image, please try again.");
		});
		return deferred.promise;
	};

	var _saveClubImage = function (clubId, file) {
		var deferred = $q.defer();
		$http({
			method: 'POST',
			url: appSettings.serviceBase + '/api/clubs/image/' + clubId,
			headers: { 'Content-Type': undefined },
			transformRequest: function (data) {
				var formData = new FormData();
				formData.append("file", data.file);

				return formData;
			},
			data: { file: file }
		}).
		success(function (data, status, headers, config) {
			deferred.resolve(data);
		}).
		error(function (data, status, headers, config) {
			deferred.reject("Could not upload club image, please try again.");
		});
		return deferred.promise;
	};

	var _saveAdImages = function (adId, fileNormal, fileSidebar, fileBackground, fileLoading) {
		var deferred = $q.defer();
		$http({
			method: 'POST',
			url: appSettings.serviceBase + '/api/ads/images/' + adId,
			headers: { 'Content-Type': undefined },
			transformRequest: function (data) {
				var formData = new FormData();
				formData.append("normal", data.fileNormal);
				formData.append("sidebar", data.fileSidebar);
				formData.append("background", data.fileBackground);
				formData.append("loading", data.fileLoading);

				return formData;
			},
			data: { fileNormal: fileNormal, fileSidebar: fileSidebar, fileBackground: fileBackground, fileLoading: fileLoading }
		}).
		success(function (data, status, headers, config) {
			deferred.resolve(data);
		}).
		error(function (data, status, headers, config) {
			deferred.reject("Could not upload ad images, please try again.");
		});
		return deferred.promise;
	};

	uploadServiceFactory.saveChallengeImage = _saveChallengeImage;
	uploadServiceFactory.saveClubImage = _saveClubImage;
	uploadServiceFactory.saveAdImages = _saveAdImages;
	return uploadServiceFactory;
}]);

services.factory('AccountService', ['dataService', function (dataService) {
	return dataService.Account;
}]);

services.factory('AuthService', ['$rootScope', '$location', '$http', '$q', '$storage', 'AccountService', 'TranslationService', 'AnalyticsService','appSettings', function ($rootScope, $location, $http, $q, $storage, AccountService, TranslationService, AnalyticsService,appSettings) {
	var authServiceFactory = {};
	var _defaultAuthData = { isAuth: false, username: "", token: "" };
	var _defaultUserData = { username: "" };
	$storage.bind($rootScope, 'username', "");
	$storage.bind($rootScope, 'authorizationData', _defaultAuthData);
	$storage.bind($rootScope, 'userData', _defaultUserData);

    var _register = function (registration) {
        _logout();

        return $http.post(appSettings.serviceBase + '/api/account/register', registration).then(function (response) {
        	TranslationService.changeLanguage(registration.language);
        	return response;
        });
    };

    var _facebook_register = function(registration,onSuccess,onError){
        return $http.post(appSettings.serviceBase + '/api/account/registerexternal', registration).then(function (response) {
            TranslationService.changeLanguage(registration.language);
            return onSuccess(response);
        },function(err){
            return onError(err.data.Message);
        });
    }

    var _facebook_connect = function (registration, onSuccess, onError) {
    	return $http.post(appSettings.serviceBase + '/api/account/connectexternal', registration).then(function (response) {
    		TranslationService.changeLanguage(registration.language);
    		return onSuccess(response);
    	}, function (err) {
    		return onError(err.data.Message);
    	});
    }

    var _login = function (login) {
        var data = "grant_type=password&username=" + login.username + "&password=" + login.password;
        var deferred = $q.defer();

        $http.post(appSettings.serviceBase + '/token', data, { headers: { 'Content-Type': 'application/x-www-form-urlencoded' } }).success(function (response) {
        	$storage.set('username', login.username);
			$storage.set('authorizationData', { isAuth: true, username: login.username, token: response.access_token });

			AccountService.get({ username: login.username },
				function (response) {
					$storage.set('userData', response);
					AnalyticsService.setUsername(response);
					TranslationService.changeLanguage(response.language);
					deferred.resolve(response);
				},
				function (err) {
					deferred.reject(err);
				});

        }).error(function (err, status) {
            _logout();
            deferred.reject(err);
        });

        return deferred.promise;
    };

    var _facebookLogin = function(accessToken,handlers){

        var data = {provider:'facebook',externalaccesstoken:accessToken};

        $http.get(appSettings.serviceBase + '/api/account/ObtainLocalAccessToken', {params:data, headers: { 'Content-Type': 'application/x-www-form-urlencoded' } }).success(function (response,status) {
            //If 204-No Content it is an unregisterd customer
            if(status == 204){
                $storage.set('fb_token', data.externalaccesstoken);
                return handlers.doRegister();
            }

            $storage.set('username', response.username);
            $storage.set('authorizationData', { isAuth: true, username: response.username, token: response.access_token });

            AccountService.get({ username: response.username },
                function (response) {
                    $storage.set('userData', response);
                    AnalyticsService.setUsername(response);
                    TranslationService.changeLanguage(response.language);
                    handlers.success(response);
                },
                function (err) {
                    handlers.error(err);
                });

        }).error(function (err, status) {
            _logout();
            handlers.error(err);
        });
    };
    var _facebookLogout = function(onSuccess,onError){
        facebookConnectPlugin.logout(function(){
            if(onSuccess)onSuccess();
        }, function(){
            if(onError)onError();
        })
    }
    var _getFacebookConnected = function (onSuccess, onError) {
    	facebookConnectPlugin.getLoginStatus(function success(resp) {
    		onSuccess(resp.status == 'connected');
    	}, function error(err) {
    		onError();
    	});
    };

    var _reset = function (resetData) {
    	return $http.post(appSettings.serviceBase + '/api/account/reset/send', resetData).then(function (response) {
    		return response;
    	});
    };

    var _logout = function () {
    	$storage.set('authorizationData', _defaultAuthData);
    	$storage.set('userData', _defaultUserData);
        _facebookLogout();
        $location.path('/login');
    };

    authServiceFactory.register = _register;
    authServiceFactory.facebook_register = _facebook_register;
    authServiceFactory.facebook_connect = _facebook_connect;
    authServiceFactory.login = _login;
    authServiceFactory.facebookLogin = _facebookLogin;
    authServiceFactory.reset = _reset;
    authServiceFactory.logout = _logout;
    authServiceFactory.getAuthentication = function () { return $storage.get('authorizationData'); };
    authServiceFactory.getUserData = function () { return $storage.get('userData'); };
    authServiceFactory.getFacebookConnected = _getFacebookConnected;
    return authServiceFactory;
}]);
